"""
Created on 27 Oct 2017

@author: Bruno Beloff (bruno.beloff@southcoastscience.com)

header:
CURLOPT_HTTPHEADER => array('Accept: application/json'),
"""

import json

from http.client import IncompleteRead

from scs_core.data.json import JSONify

from scs_core.client.http_client import HTTPClient
from scs_core.client.http_exception import HTTPException
from scs_core.client.http_status import HTTPStatus

from scs_philips_hue.client.client_exception import ClientException


# --------------------------------------------------------------------------------------------------------------------

class RESTClient(object):
    """
    classdocs
    """

    __HEADER_ACCEPT = "application/json"

    # ----------------------------------------------------------------------------------------------------------------

    def __init__(self):
        """
        Constructor
        """
        self.__http_client = HTTPClient()
        self.__username = None


    # ----------------------------------------------------------------------------------------------------------------

    def connect(self, host, username, timeout=None):
        self.__http_client.connect(host, secure=False, timeout=timeout)
        self.__username = username


    def close(self):
        self.__http_client.close()


    # ----------------------------------------------------------------------------------------------------------------

    def get(self, path, params=None):
        # print("RESTClient.get: path: %s" % self.__http_path(path))

        try:
            response_jstr = self.__http_client.get(self.__http_path(path), params, self.__headers)

        except (ConnectionRefusedError, IncompleteRead):
            return None

        except HTTPException as exc:
            if exc.status == HTTPStatus.NOT_FOUND:
                return None
            else:
                raise ClientException.construct(exc) from exc

        try:
            response = json.loads(response_jstr)
        except ValueError:
            response = None

        return response


    def post(self, path, payload_jdict):
        payload_jstr = JSONify.dumps(payload_jdict)

        try:
            response_jstr = self.__http_client.post(self.__http_path(path), payload_jstr, self.__headers)

        except (ConnectionRefusedError, IncompleteRead):
            return None

        except HTTPException as ex:
            raise ClientException.construct(ex) from ex

        try:
            response = json.loads(response_jstr)
        except ValueError:
            response = None

        return response


    def put(self, path, payload_jdict):                # TODO: make the jdict here?
        payload_jstr = JSONify.dumps(payload_jdict)

        # print("RESTClient.put: path: %s payload: %s" % (path, payload_jstr))

        try:
            response_jstr = self.__http_client.put(self.__http_path(path), payload_jstr, self.__headers)

        except (ConnectionRefusedError, IncompleteRead):
            return None

        except HTTPException as ex:
            raise ClientException.construct(ex) from ex

        try:
            response = json.loads(response_jstr)
        except ValueError:
            response = None

        return response


    def delete(self, path):
        try:
            response_jstr = self.__http_client.delete(self.__http_path(path), self.__headers)

        except (ConnectionRefusedError, IncompleteRead):
            return None

        except HTTPException as ex:
            raise ClientException.construct(ex) from ex

        try:
            response = json.loads(response_jstr)
        except ValueError:
            response = None

        return response


    # ----------------------------------------------------------------------------------------------------------------

    def __http_path(self, path):
        if self.__username is None:
            return '/api' + path

        return '/api/' + self.__username + path


    @property
    def __headers(self):
        return {"Accept": self.__HEADER_ACCEPT}


    # ----------------------------------------------------------------------------------------------------------------

    def __str__(self, *args, **kwargs):
        return "RESTClient:{username:%s}" % self.__username



"""
Created on 17 Apr 2023

@author: Bruno Beloff (bruno.beloff@southcoastscience.com)
"""

from scs_core.sys.logging import Logging

from scs_philips_hue.config.bridge_address import BridgeAddress, BridgeAddressSet
from scs_philips_hue.discovery.discovery import Discovery
from scs_philips_hue.manager.bridge_manager import BridgeManager, BridgeManagerGroup


# --------------------------------------------------------------------------------------------------------------------

class BridgeBuilder(object):
    """
    classdocs
    """

    # ----------------------------------------------------------------------------------------------------------------

    def __init__(self, host):
        self.__host = host                                          # PersistenceManager
        self.__logger = Logging.getLogger()                         # logger


    # ----------------------------------------------------------------------------------------------------------------

    def construct_all(self, credentials_set):
        bridge_managers = {}

        for bridge_name, credentials in credentials_set.sorted_credentials.items():
            manager = self.construct_manager_for_credentials(credentials)

            bridge_managers[bridge_name] = manager                  # includes 'None' managers

        return BridgeManagerGroup(bridge_managers)


    def construct_dict_for_credentials(self, credentials):
        manager = self.construct_manager_for_credentials(credentials)

        return {} if manager is None else {credentials.bridge_name: manager}


    def construct_manager_for_credentials(self, credentials):
        address_set = BridgeAddressSet.load(self.__host, skeleton=True)

        self.__logger.info("looking for '%s'..." % credentials.bridge_name)

        # cached address..
        manager = self.__find_by_cached_address(address_set, credentials)

        # find bridge...
        if manager is None:
            manager = self.__find_by_scan(address_set, credentials)

        self.__logger.info("%s: %s" % (credentials.bridge_name, manager))

        return manager


    # ----------------------------------------------------------------------------------------------------------------

    def __find_by_cached_address(self, address_set, credentials):
        try:
            address = address_set.address(credentials.bridge_name)

            if BridgeManager.is_bridge(address.ipv4.dot_decimal()):
                return BridgeManager(address.ipv4.dot_decimal(), credentials.username)

            # remove invalid address...
            self.__logger.info("invalid cached IP address")
            address_set.delete(credentials.bridge_name)
            address_set.save(self.__host)

        except KeyError:
            self.__logger.info("no cached IP address")

        return None


    def __find_by_scan(self, address_set, credentials):
        discovery = Discovery(self.__host)
        bridge = discovery.find(credentials)

        if bridge is None:
            self.__logger.error("bridge '%s' not found." % credentials.bridge_name)
            return None
            # exit(1)

        if bridge.ip_address is None:
            self.__logger.error("stored credentials are not valid for bridge '%s'." % credentials.bridge_name)
            return None
            # exit(1)

        # add address...
        address_set.add(BridgeAddress.construct(credentials.bridge_name, bridge.ip_address))
        address_set.save(self.__host)

        return BridgeManager(bridge.ip_address, credentials.username)


    # ----------------------------------------------------------------------------------------------------------------

    def __str__(self, *args, **kwargs):
        return "BridgeBuilder:{}"



"""
Created on 3 Nov 2017

@author: Bruno Beloff (bruno.beloff@southcoastscience.com)
"""

import optparse

from scs_philips_hue import version


# --------------------------------------------------------------------------------------------------------------------

class CmdUser(object):
    """unix command line handler"""

    def __init__(self):
        """
        Constructor
        """
        self.__parser = optparse.OptionParser(usage="%prog { -l | -r USER } [-i INDENT] [-v] BRIDGE_NAME",
                                              version=version())

        # operations...
        self.__parser.add_option("--list", "-l", action="store_true", dest="list",
                                 help="list all users")

        self.__parser.add_option("--remove", "-r", type="string", action="store", dest="remove",
                                 help="remove the user from the bridge")

        # output...
        self.__parser.add_option("--indent", "-i", type="int", action="store", dest="indent",
                                 help="pretty-print the output with INDENT")

        self.__parser.add_option("--verbose", "-v", action="store_true", dest="verbose", default=False,
                                 help="report narrative to stderr")

        self.__opts, self.__args = self.__parser.parse_args()


    # ----------------------------------------------------------------------------------------------------------------

    def is_valid(self):
        if self.bridge_name is None:
            return False

        if self.remove is None and not self.list:
            return False

        if self.remove is not None and self.list:
            return False

        return True


    # ----------------------------------------------------------------------------------------------------------------

    @property
    def list(self):
        return self.__opts.list


    @property
    def remove(self):
        return self.__opts.remove


    @property
    def indent(self):
        return self.__opts.indent


    @property
    def verbose(self):
        return self.__opts.verbose


    @property
    def bridge_name(self):
        return self.__args[0] if self.__args else None


    # ----------------------------------------------------------------------------------------------------------------

    def print_help(self, file):
        self.__parser.print_help(file)


    def __str__(self, *args, **kwargs):
        return "CmdUser:{bridge_name:%s, list:%s, remove:%s, indent:%s, verbose:%s}" % \
            (self.bridge_name, self.list, self.remove, self.indent, self.verbose)



#!/usr/bin/env python3

"""
Created on 16 Mar 2018

@author: Bruno Beloff (bruno.beloff@southcoastscience.com)

source repo: scs_philips_hue

DESCRIPTION
The chroma_conf utility is used to specify the parameters of a mapping from environmental data domain values to
locations in a chromaticity space. In addition to chromaticity mapping, the configuration includes lamp brightness and
transition time.

A chromaticity chart is available at: https://developers.meethue.com/documentation/core-concepts

The chroma_conf utility works in conjunction with a chroma path: the path specifies two or more ordered points in
chroma space, indicating a progression through different colours as a domain value changes. The chroma path
specifies the shape of this route, the chroma conf specifies the minimum and maximum domain values associated
with a progression though the chroma space.

Note that - currently - there is no command line utility to edit chroma paths, and only default paths can be used.

SYNOPSIS
chroma_conf.py [-c CHANNEL { [-p PATH_NAME] [-l DOMAIN_MIN] [-u DOMAIN_MAX] [-b BRIGHTNESS] [-t TRANSITION] | -r }]
[-i INDENT] [-v]

EXAMPLES
./chroma_conf.py -vi4 -c preston-circus-pm10 -p risk_level -l 0 -u 50 -b 254 -t 9

FILES
~/SCS/hue/chroma_conf_set.json

DOCUMENT EXAMPLE
{
    "preston_circus": {
        "path-name": "risk_level",
        "domain-min": 0.0,
        "domain-max": 50.0,
        "brightness": 254,
        "transition-time": 9
    }
}

SEE ALSO
scs_philips_hue/chroma

RESOURCES
https://en.wikipedia.org/wiki/Chromaticity
https://developers.meethue.com/documentation/core-concepts
"""

import sys

from scs_core.data.json import JSONify
from scs_core.sys.logging import Logging

from scs_host.sys.host import Host

from scs_philips_hue.cmd.cmd_chroma_conf import CmdChromaConf
from scs_philips_hue.config.chroma_conf import ChromaConfSet


# --------------------------------------------------------------------------------------------------------------------

if __name__ == '__main__':

    # ----------------------------------------------------------------------------------------------------------------
    # cmd...

    cmd = CmdChromaConf()

    if not cmd.is_valid():
        cmd.print_help(sys.stderr)
        exit(2)

    Logging.config('chroma_conf', verbose=cmd.verbose)
    logger = Logging.getLogger()

    logger.info(cmd)


    # ----------------------------------------------------------------------------------------------------------------
    # resources...

    # ChromaConf...
    chromas = ChromaConfSet.load(Host, skeleton=True)


    # ----------------------------------------------------------------------------------------------------------------
    # run...

    if cmd.set():
        chroma = chromas.conf(cmd.channel)

        if chroma is None:
            if not cmd.is_complete():
                logger.error("no configuration is stored - you must therefore set all fields.")
                exit(2)

            chromas.add(cmd.channel, cmd.path_name, cmd.domain_min, cmd.domain_max, cmd.brightness, cmd.transition_time)

        else:
            path_name = chroma.path_name if cmd.path_name is None else cmd.path_name
            domain_min = chroma.domain_min if cmd.domain_min is None else cmd.domain_min
            domain_max = chroma.domain_max if cmd.domain_max is None else cmd.domain_max
            brightness = chroma.brightness if cmd.brightness is None else cmd.brightness
            transition_time = chroma.transition_time if cmd.transition_time is None else cmd.transition_time

            chromas.add(cmd.channel, path_name, domain_min, domain_max, brightness, transition_time)

        chromas.save(Host)

    if cmd.remove:
        chromas.remove(cmd.channel)
        chromas.save(Host)


    if chromas:
        print(JSONify.dumps(chromas, indent=cmd.indent))



"""
Created on 28 Oct 2017

@author: Bruno Beloff (bruno.beloff@southcoastscience.com)

example:
{"internet": "connected", "remoteaccess": "connected", "time": "connected", "swupdate": "connected"}
"""

from collections import OrderedDict

from scs_core.data.json import JSONable


# --------------------------------------------------------------------------------------------------------------------

class InternetServices(JSONable):
    """
    classdocs
    """

    # ----------------------------------------------------------------------------------------------------------------

    @classmethod
    def construct_from_jdict(cls, jdict):
        if not jdict:
            return None

        internet = jdict.get('internet')
        remote_access = jdict.get('remoteaccess')
        time = jdict.get('time')
        sw_update = jdict.get('swupdate')

        return InternetServices(internet, remote_access, time, sw_update)


    # ----------------------------------------------------------------------------------------------------------------

    def __init__(self, internet, remote_access, time, sw_update):
        """
        Constructor
        """
        self.__internet = internet                      # string
        self.__remote_access = remote_access            # string
        self.__time = time                              # string
        self.__sw_update = sw_update                    # string


    # ----------------------------------------------------------------------------------------------------------------

    def as_json(self):
        jdict = OrderedDict()

        jdict['internet'] = self.internet
        jdict['remoteaccess'] = self.remote_access
        jdict['time'] = self.time
        jdict['swupdate'] = self.sw_update

        return jdict


    # ----------------------------------------------------------------------------------------------------------------

    @property
    def internet(self):
        return self.__internet


    @property
    def remote_access(self):
        return self.__remote_access


    @property
    def time(self):
        return self.__time


    @property
    def sw_update(self):
        return self.__sw_update


    # ----------------------------------------------------------------------------------------------------------------

    def __str__(self, *args, **kwargs):
        return "InternetServices:{internet:%s, remote_access:%s, time:%s, sw_update:%s}" %  \
               (self.internet, self.remote_access, self.time, self.sw_update)



#!/usr/bin/env python3

"""
Created on 11 Nov 2017

@author: Bruno Beloff (bruno.beloff@southcoastscience.com)

source repo: scs_philips_hue

DESCRIPTION
The bridge utility is used to interrogate and update the Philips Hue Bridge device.

If a bridge address has been stored, this is used to find the bridge. Otherwise a UPnP or IP scan is attempted.

SYNOPSIS
bridge.py [-p PORTAL_SERVICES] [-c CHECK_UPDATE] [-u DO_UPDATE] [-z CHANNEL] [-i INDENT] [-v] BRIDGE_NAME

EXAMPLES
./bridge.py -vi4 hue-br1-002

FILES
~/SCS/hue/bridge_address_set.json
~/SCS/hue/bridge_credentials_set.json

DOCUMENT EXAMPLE
{"bridge-id": "001788fffe795620", "username": "TIYoqrnwkvyODu8xE9zvRxjIJSRSde0qUzUqqIr7"}

SEE ALSO
scs_philips_hue/bridge_address
scs_philips_hue/join
scs_philips_hue/user

RESOURCES
https://developers.meethue.com/content/configuring-hue-without-phone-app-unable-update-software
"""

import sys

from scs_core.client.http_exception import HTTPException
from scs_core.client.network import Network
from scs_core.client.resource_unavailable_exception import ResourceUnavailableException

from scs_core.data.json import JSONify

from scs_core.sys.logging import Logging

from scs_host.sys.host import Host

from scs_philips_hue.cmd.cmd_bridge import CmdBridge

from scs_philips_hue.config.bridge_credentials import BridgeCredentialsSet

from scs_philips_hue.data.bridge.bridge_config import BridgeConfig
from scs_philips_hue.data.bridge.sw_update import SWUpdate

from scs_philips_hue.manager.bridge_builder import BridgeBuilder


# --------------------------------------------------------------------------------------------------------------------

if __name__ == '__main__':

    response = None

    # ----------------------------------------------------------------------------------------------------------------
    # cmd...

    cmd = CmdBridge()

    if not cmd.is_valid():
        cmd.print_help(sys.stderr)
        exit(2)

    Logging.config('bridge', verbose=cmd.verbose)
    logger = Logging.getLogger()

    logger.info(cmd)

    try:
        # ------------------------------------------------------------------------------------------------------------
        # check...

        if not Network.is_available():
            logger.info("waiting for network.")
            Network.wait()


        # ------------------------------------------------------------------------------------------------------------
        # resources...

        # BridgeCredentials...
        credentials_set = BridgeCredentialsSet.load(Host, skeleton=True)

        if len(credentials_set) < 1:
            logger.error("BridgeCredentials not available.")
            exit(1)

        try:
            credentials = credentials_set.credentials(cmd.bridge_name)
            logger.info(credentials)

        except KeyError:
            logger.error("no stored credentials for bridge '%s'." % cmd.bridge_name)
            exit(1)

        # manager...
        manager = BridgeBuilder(Host).construct_manager_for_credentials(credentials)


        # ------------------------------------------------------------------------------------------------------------
        # run...

        # initial state...
        config = manager.find()

        # portal services...
        if cmd.portal_services:
            config = BridgeConfig(portal_services=cmd.portal_services)
            response = manager.set_config(config)

        # check for update...
        if cmd.check_update:
            config = BridgeConfig(sw_update=SWUpdate(check_for_update=cmd.check_update))
            response = manager.set_config(config)

        # do update...
        if cmd.do_update:
            if config.sw_update.update_state != SWUpdate.UPDATE_AVAILABLE:
                logger.error("no software update available.")
                exit(1)

            config = BridgeConfig(sw_update=SWUpdate(update_state=SWUpdate.UPDATE_PERFORM))
            response = manager.set_config(config)

        # zigbee...
        if cmd.zigbee_channel:
            config = BridgeConfig(zigbee_channel=cmd.zigbee_channel)
            response = manager.set_config(config)

        if response:
            logger.info(response)

        config = manager.find()
        print(JSONify.dumps(config, indent=cmd.indent))


    # ----------------------------------------------------------------------------------------------------------------
    # end...

    except KeyboardInterrupt:
        print(file=sys.stderr)

    except (ConnectionError, HTTPException, ResourceUnavailableException) as ex:
        logger.error(repr(ex))
        exit(1)

    except TimeoutError:
        logger.error("Timeout")
        exit(1)



"""
Created on 16 Mar 2018

@author: Bruno Beloff (bruno.beloff@southcoastscience.com)

document example:
{"NO2": {"path-name": "risk-level", "domain-min": 0.0, "domain-max": 50.0, "brightness": 254, "transition-time": 9},
"PM10": {"path-name": "risk-level", "domain-min": 0.0, "domain-max": 100.0, "brightness": 254, "transition-time": 9}}
"""

from collections import OrderedDict

from scs_core.data.datum import Datum
from scs_core.data.json import JSONable

from scs_philips_hue.config.chroma_path import ChromaPath
from scs_philips_hue.config.conf_set import ConfSet

from scs_philips_hue.data.light.chroma import ChromaMapping


# --------------------------------------------------------------------------------------------------------------------

class ChromaConfSet(ConfSet):
    """
    classdocs
    """

    __FILENAME =        "chroma_conf_set.json"

    @classmethod
    def persistence_location(cls):
        return cls.hue_dir(), cls.__FILENAME


    # ----------------------------------------------------------------------------------------------------------------

    @classmethod
    def construct_from_jdict(cls, jdict, skeleton=False):
        if not jdict:
            return cls({}) if skeleton else None

        confs = {}

        for channel, conf_jdict in jdict.items():
            confs[channel] = ChromaConf.construct_from_jdict(conf_jdict)

        return cls(confs)


    # ----------------------------------------------------------------------------------------------------------------

    def __init__(self, confs):
        """
        Constructor
        """
        super().__init__(confs)


    # ----------------------------------------------------------------------------------------------------------------

    def add(self, channel, path_name, domain_min, domain_max, brightness, transition_time):
        self._confs[channel] = ChromaConf(path_name, domain_min, domain_max, brightness, transition_time)


# --------------------------------------------------------------------------------------------------------------------

class ChromaConf(JSONable):
    """
    classdocs
    """

    # ----------------------------------------------------------------------------------------------------------------

    @classmethod
    def construct_from_jdict(cls, jdict):
        if not jdict:
            return None

        path_name = jdict.get('path-name')

        domain_min = jdict.get('domain-min')
        domain_max = jdict.get('domain-max')

        brightness = jdict.get('brightness')
        transition_time = jdict.get('transition-time')

        return cls(path_name, domain_min, domain_max, brightness, transition_time)


    # ----------------------------------------------------------------------------------------------------------------

    def __init__(self, path_name, domain_min, domain_max, brightness, transition_time):
        """
        Constructor
        """
        self.__path_name = path_name                                # string

        self.__domain_min = domain_min                              # float
        self.__domain_max = domain_max                              # float

        self.__brightness = Datum.int(brightness)                   # int
        self.__transition_time = Datum.int(transition_time)         # int


    # ----------------------------------------------------------------------------------------------------------------

    def path(self):
        return ChromaPath.retrieve(self.path_name)


    def mapping(self, path):
        return ChromaMapping.construct(self, path)


    # ----------------------------------------------------------------------------------------------------------------

    def as_json(self):
        jdict = OrderedDict()

        jdict['path-name'] = self.path_name

        jdict['domain-min'] = self.domain_min
        jdict['domain-max'] = self.domain_max

        jdict['brightness'] = self.brightness
        jdict['transition-time'] = self.transition_time

        return jdict


    # ----------------------------------------------------------------------------------------------------------------

    @property
    def path_name(self):
        return self.__path_name


    @property
    def domain_min(self):
        return self.__domain_min


    @property
    def domain_max(self):
        return self.__domain_max


    @property
    def brightness(self):
        return self.__brightness


    @property
    def transition_time(self):
        return self.__transition_time


    # ----------------------------------------------------------------------------------------------------------------

    def __str__(self, *args, **kwargs):
        return "ChromaConf:{path_name:%s, domain_min:%s, domain_max:%s, brightness:%s, transition_time:%s}" % \
               (self.path_name, self.domain_min, self.domain_max, self.brightness, self.transition_time)



"""
Created on 9 Nov 2023

@author: Bruno Beloff (bruno.beloff@southcoastscience.com)
"""

import time

from multiprocessing import Manager

from scs_core.sync.synchronised_process import SynchronisedProcess
from scs_core.sys.logging import Logging

from scs_philips_hue.manager.bridge_builder import BridgeBuilder
from scs_philips_hue.manager.bridge_manager import BridgeManagerGroup


# --------------------------------------------------------------------------------------------------------------------

class BridgeMonitor(SynchronisedProcess):
    """
    classdocs
    """
    __MONITOR_INTERVAL =        60              # seconds

    # ----------------------------------------------------------------------------------------------------------------

    @classmethod
    def construct(cls, host, credentials_set):
        return cls(BridgeBuilder(host), credentials_set)


    # ----------------------------------------------------------------------------------------------------------------

    def __init__(self, builder: BridgeBuilder, credentials_set):
        """
        Constructor
        """
        self.__logger = Logging.getLogger()
        self.__logging_specification = Logging.specification()

        manager = Manager()

        SynchronisedProcess.__init__(self, value=manager.list())

        self.__builder = builder                                        # BridgeBuilder
        self.__credentials_set = credentials_set                        # BridgeCredentialsSet


    # ----------------------------------------------------------------------------------------------------------------
    # SynchronisedProcess implementation...

    def run(self):
        Logging.replicate(self.__logging_specification)

        try:
            while True:
                group = self.__builder.construct_all(self.__credentials_set)

                # report...
                with self._lock:
                    group.as_list(self._value)

                time.sleep(self.__MONITOR_INTERVAL)

        except (KeyboardInterrupt, SystemExit):
            return


    # ----------------------------------------------------------------------------------------------------------------
    # data retrieval for client process...

    @property
    def bridge_manager_group(self):
        with self._lock:
            group = BridgeManagerGroup.construct_from_jdict(self._value)

        return group


    # ----------------------------------------------------------------------------------------------------------------

    def __str__(self, *args, **kwargs):
        return "BridgeMonitor:{builder:%s, credentials_set:%s}" % (self.__builder, self.__credentials_set)



#!/usr/bin/env python3

"""
Created on 22 Nov 2023

@author: Bruno Beloff (bruno.beloff@southcoastscience.com)

source repo: scs_philips_hue

DESCRIPTION
The upnp_conf utility is used to enable or disable networking UPnP discovery for all relevant SCS Philips Hue utilities.

Note: by default, UPnP discovery is not enabled.

SYNOPSIS
Usage: upnp_conf.py [-e { 0 | 1 }] [-i INDENT] [-v]

EXAMPLES
./upnp_conf.py -v

FILES
~/SCS/hue/upnp_conf.json

DOCUMENT EXAMPLE
{"upnp-enabled": false}

SEE ALSO
scs_philips_hue/bridge
scs_philips_hue/join
scs_philips_hue/light
scs_philips_hue/user
"""

import sys

from scs_core.data.json import JSONify
from scs_core.sys.logging import Logging

from scs_host.sys.host import Host

from scs_philips_hue.cmd.cmd_upnp_conf import CmdUPnPConf
from scs_philips_hue.discovery.upnp_conf import UPnPConf


# --------------------------------------------------------------------------------------------------------------------

if __name__ == '__main__':

    bridge = None

    # ----------------------------------------------------------------------------------------------------------------
    # cmd...

    cmd = CmdUPnPConf()

    if not cmd.is_valid():
        cmd.print_help(sys.stderr)
        exit(2)

    Logging.config('upnp_conf', verbose=cmd.verbose)
    logger = Logging.getLogger()

    logger.info(cmd)

    try:
        # ----------------------------------------------------------------------------------------------------------------
        # resources...

        config = UPnPConf.load(Host, skeleton=True)


        # ----------------------------------------------------------------------------------------------------------------
        # run...

        if cmd.set():
            config = UPnPConf(cmd.enable)
            config.save(Host)

        # report...
        print(JSONify.dumps(config, indent=cmd.indent))


    # ----------------------------------------------------------------------------------------------------------------
    # end...

    except KeyboardInterrupt:
        print(file=sys.stderr)




"""
Created on 28 Oct 2017

@author: Bruno Beloff (bruno.beloff@southcoastscience.com)

example:
{
    "updatestate": 0,
    "checkforupdate": false,
    "devicetypes": {"device_types": false, "lights": [], "sensors": []},
    "url": "",
    "text": "",
    "notify": false
}
"""

from collections import OrderedDict

from scs_core.data.datum import Datum
from scs_core.data.json import JSONable


# --------------------------------------------------------------------------------------------------------------------

class SWUpdate(JSONable):
    """
    classdocs
    """

    UPDATE_UNAVAILABLE =        1
    UPDATE_AVAILABLE =          2
    UPDATE_PERFORM =            3

    # ----------------------------------------------------------------------------------------------------------------

    @classmethod
    def construct_from_jdict(cls, jdict):
        if not jdict:
            return None

        update_state = jdict.get('updatestate')
        check_for_update = jdict.get('checkforupdate')
        device_types = DeviceTypes.construct_from_jdict(jdict.get('devicetypes'))

        url = jdict.get('url')
        text = jdict.get('text')
        notify = jdict.get('notify')

        return SWUpdate(update_state=update_state, check_for_update=check_for_update, device_types=device_types,
                        url=url, text=text, notify=notify)


    # ----------------------------------------------------------------------------------------------------------------

    def __init__(self, update_state=None, check_for_update=None, device_types=None,
                 url=None, text=None, notify=None):
        """
        Constructor
        """
        self.__update_state = Datum.int(update_state)                   # int
        self.__check_for_update = Datum.bool(check_for_update)          # bool
        self.__device_types = device_types                              # DeviceTypes

        self.__url = url                                                # string

        self.__text = text                                              # string
        self.__notify = Datum.bool(notify)                              # bool


    # ----------------------------------------------------------------------------------------------------------------

    def as_json(self):
        jdict = OrderedDict()

        if self.update_state is not None:
            jdict['updatestate'] = self.update_state

        if self.check_for_update is not None:
            jdict['checkforupdate'] = self.check_for_update

        if self.device_types is not None:
            jdict['devicetypes'] = self.device_types

        if self.url is not None:
            jdict['url'] = self.url

        if self.text is not None:
            jdict['text'] = self.text

        if self.notify is not None:
            jdict['notify'] = self.notify

        return jdict


    # ----------------------------------------------------------------------------------------------------------------

    @property
    def update_state(self):
        return self.__update_state


    @property
    def check_for_update(self):
        return self.__check_for_update


    @property
    def device_types(self):
        return self.__device_types


    @property
    def url(self):
        return self.__url


    @property
    def text(self):
        return self.__text


    @property
    def notify(self):
        return self.__notify


    # ----------------------------------------------------------------------------------------------------------------

    def __str__(self, *args, **kwargs):
        return "SWUpdate:{update_state:%s, check_for_update:%s, device_types:%s, url:%s, text:%s, notify:%s}" %  \
               (self.update_state, self.check_for_update, self.device_types, self.url, self.text, self.notify)


# --------------------------------------------------------------------------------------------------------------------

class DeviceTypes(JSONable):
    """
    classdocs
    """

    # ----------------------------------------------------------------------------------------------------------------

    @classmethod
    def construct_from_jdict(cls, jdict):
        if not jdict:
            return None

        bridge = jdict.get('bridge')
        lights = jdict.get('lights')
        sensors = jdict.get('sensors')

        return DeviceTypes(bridge, lights, sensors)


    # ----------------------------------------------------------------------------------------------------------------

    def __init__(self, bridge, lights, sensors):
        """
        Constructor
        """
        self.__bridge = bool(bridge)                # bool
        self.__lights = lights                      # array
        self.__sensors = sensors                    # array


    # ----------------------------------------------------------------------------------------------------------------

    def as_json(self):
        jdict = OrderedDict()

        jdict['bridge'] = self.bridge
        jdict['lights'] = self.lights
        jdict['sensors'] = self.sensors

        return jdict


    # ----------------------------------------------------------------------------------------------------------------

    @property
    def bridge(self):
        return self.__bridge


    @property
    def lights(self):
        return self.__lights


    @property
    def sensors(self):
        return self.__sensors


    # ----------------------------------------------------------------------------------------------------------------

    def __str__(self, *args, **kwargs):
        return "DeviceTypes:{bridge:%s, lights:%s, sensors:%s}" %  \
               (self.bridge, self.lights, self.sensors)



"""
Created on 28 Oct 2017

@author: Bruno Beloff (bruno.beloff@southcoastscience.com)

example:
{
    "state": {
        "on": true,
        "bri": 254,
        "hue": 8418,
        "sat": 140,
        "effect": "none",
        "xy": [0.4573, 0.41],
        "ct": 366,
        "alert": "select",
        "colormode": "ct",
        "reachable": true
    },
    "swupdate": {
        "state": "noupdates",
        "lastinstall": null
    },
    "light_type": "Extended color light",
    "name": "Hue color lamp 1",
    "modelid": "LCT015",
    "manufacturername": "Philips",
    "uniqueid": "00:17:88:01:03:54:25:66-0b",
    "swversion": "1.19.0_r19755",
    "swconfigid": "A724919D",
    "productid": "Philips-LCT015-1-A19ECLv5"
}
"""

# from scs_core.data.json import JSONify

from collections import OrderedDict

from scs_core.data.json import JSONable

from scs_philips_hue.data.light.light_state import ReportedLightState
from scs_philips_hue.data.light.sw_update import SWUpdate


# --------------------------------------------------------------------------------------------------------------------

class Light(JSONable):
    """
    classdocs
    """

    # ----------------------------------------------------------------------------------------------------------------

    @classmethod
    def construct_from_jdict(cls, jdict):
        # print(JSONify.dumps(jdict))

        if not jdict:
            return None

        state = ReportedLightState.construct_from_jdict(jdict.get('state'))
        swupdate = SWUpdate.construct_from_jdict(jdict.get('swupdate'))

        light_type = jdict.get('type')
        name = jdict.get('name')
        model_id = jdict.get('modelid')
        manufacturer_name = jdict.get('manufacturername')
        unique_id = jdict.get('uniqueid')

        sw_version = jdict.get('swversion')
        sw_config_id = jdict.get('swconfigid')
        product_id = jdict.get('productid')

        return cls(state, swupdate, light_type, name, model_id, manufacturer_name, unique_id,
                   sw_version, sw_config_id, product_id)


    # ----------------------------------------------------------------------------------------------------------------

    def __init__(self, state, swupdate, light_type, name, model_id, manufacturer_name,
                 unique_id, sw_version, sw_config_id, product_id):
        """
        Constructor
        """
        self.__state = state                                    # ReportedLightState
        self.__swupdate = swupdate                              # SWUpdate

        self.__light_type = light_type                          # string
        self.__name = name                                      # string
        self.__model_id = model_id                              # string
        self.__manufacturer_name = manufacturer_name            # string
        self.__unique_id = unique_id                            # string

        self.__sw_version = sw_version                          # string
        self.__sw_config_id = sw_config_id                      # string
        self.__product_id = product_id                          # string


    # ----------------------------------------------------------------------------------------------------------------

    def as_json(self):
        jdict = OrderedDict()

        jdict['state'] = self.state
        jdict['swupdate'] = self.swupdate

        jdict['type'] = self.light_type
        jdict['name'] = self.name
        jdict['modelid'] = self.model_id
        jdict['manufacturername'] = self.manufacturer_name
        jdict['uniqueid'] = self.unique_id

        jdict['swversion'] = self.sw_version
        jdict['swconfigid'] = self.sw_config_id
        jdict['productid'] = self.product_id

        return jdict


    # ----------------------------------------------------------------------------------------------------------------

    @property
    def state(self):
        return self.__state


    @property
    def swupdate(self):
        return self.__swupdate


    @property
    def light_type(self):
        return self.__light_type


    @property
    def name(self):
        return self.__name


    @property
    def model_id(self):
        return self.__model_id


    @property
    def manufacturer_name(self):
        return self.__manufacturer_name


    @property
    def unique_id(self):
        return self.__unique_id


    @property
    def sw_version(self):
        return self.__sw_version


    @property
    def sw_config_id(self):
        return self.__sw_config_id


    @property
    def product_id(self):
        return self.__product_id


    # ----------------------------------------------------------------------------------------------------------------

    def __str__(self, *args, **kwargs):
        return "Light:{state:%s, swupdate:%s, light_type:%s, name:%s, model_id:%s, manufacturer_name:%s, " \
               "unique_id:%s, sw_version:%s, sw_config_id:%s, product_id:%s}" %  \
               (self.state, self.swupdate, self.light_type, self.name, self.model_id, self.manufacturer_name,
                self.unique_id, self.sw_version, self.sw_config_id, self.product_id)


# --------------------------------------------------------------------------------------------------------------------

class LightListEntry(JSONable):
    """
    classdocs
    """

    # ----------------------------------------------------------------------------------------------------------------

    @classmethod
    def construct_from_jdict(cls, index, jdict):
        if not jdict:
            return None

        light = Light.construct_from_jdict(jdict)

        return LightListEntry(index, light)


    # ----------------------------------------------------------------------------------------------------------------

    def __init__(self, index, light):
        """
        Constructor
        """
        self.__index = index                            # index
        self.__light = light                            # Light


    # ----------------------------------------------------------------------------------------------------------------

    def as_json(self):
        jdict = OrderedDict()

        jdict[self.index] = self.light

        return jdict


    # ----------------------------------------------------------------------------------------------------------------

    @property
    def index(self):
        return self.__index


    @property
    def light(self):
        return self.__light


    # ----------------------------------------------------------------------------------------------------------------

    def __str__(self, *args, **kwargs):
        return "LightListEntry:{index:%s, light:%s}" %  (self.index, self.light)



"""
Created on 25 Apr 2017

@author: Bruno Beloff (bruno.beloff@southcoastscience.com)
"""

import optparse

from scs_philips_hue import version


# --------------------------------------------------------------------------------------------------------------------

class CmdNode(object):
    """unix command line handler"""

    def __init__(self):
        """
        Constructor
        """
        self.__parser = optparse.OptionParser(usage="%prog { -c | -t NAME TOPIC_PATH DOMAIN_NODE } [-i] [-v]",
                                              version=version())

        # input...
        self.__parser.add_option("--conf", "-c", action="store_true", dest="use_domain_conf", default=False,
                                 help="get topic.node from the domain conf")

        self.__parser.add_option("--topic", "-t", type="string", nargs=3, action="store", dest="topic_path",
                                 help="use the given name, topic and node")

        # output...
        self.__parser.add_option("--verbose", "-v", action="store_true", dest="verbose", default=False,
                                 help="report narrative to stderr")

        self.__opts, self.__args = self.__parser.parse_args()


    # ----------------------------------------------------------------------------------------------------------------

    def is_valid(self):
        if self.use_domain_conf and self.topic_path is not None:
            return False

        if not self.use_domain_conf and self.topic_path is None:
            return False

        return True


    # ----------------------------------------------------------------------------------------------------------------

    @property
    def use_domain_conf(self):
        return self.__opts.use_domain_conf


    @property
    def topic_path(self):
        return self.__opts.topic_path


    @property
    def verbose(self):
        return self.__opts.verbose


    # ----------------------------------------------------------------------------------------------------------------

    def print_help(self, file):
        self.__parser.print_help(file)


    def __str__(self, *args, **kwargs):
        return "CmdNode:{use_domain_conf:%s, topic_path:%s, verbose:%s}" %  \
               (self.use_domain_conf, self.topic_path, self.verbose)



"""
Created on 16 Mar 2018

@author: Bruno Beloff (bruno.beloff@southcoastscience.com)

document example:
{"topic-path": "/orgs/south-coast-science-demo/brighton/loc/1/particulates", "document-node": "val.pm10"}
"""

import optparse

from scs_philips_hue import version


# --------------------------------------------------------------------------------------------------------------------

class CmdDomainConf(object):
    """unix command line handler"""

    def __init__(self):
        """
        Constructor
        """
        self.__parser = optparse.OptionParser(usage="%prog [-c CHANNEL { -a TOPIC_PATH DOMAIN_NODE | -r }] "
                                                    "[-i INDENT] [-v]", version=version())

        # configuration...
        self.__parser.add_option("--channel", "-c", type="string", action="store", dest="channel",
                                 help="the name of the information channel")

        # operations...
        self.__parser.add_option("--add", "-a", type="string", nargs=2, action="store", dest="add",
                                 help="add the domain configuration")

        self.__parser.add_option("--remove", "-r", action="store_true", dest="remove", default=False,
                                 help="remove the domain configuration")

        # output...
        self.__parser.add_option("--indent", "-i", type="int", action="store", dest="indent",
                                 help="pretty-print the output with INDENT")

        self.__parser.add_option("--verbose", "-v", action="store_true", dest="verbose", default=False,
                                 help="report narrative to stderr")

        self.__opts, self.__args = self.__parser.parse_args()


    # ----------------------------------------------------------------------------------------------------------------

    def is_valid(self):
        if self.add is not None and self.remove:
            return False

        if self.set() and self.channel is None:
            return False

        if not self.set() and self.channel is not None:
            return False

        return True


    def set(self):
        return self.add is not None or self.remove


    # ----------------------------------------------------------------------------------------------------------------

    @property
    def channel(self):
        return self.__opts.channel


    @property
    def add(self):
        return self.__opts.add


    @property
    def remove(self):
        return self.__opts.remove


    @property
    def indent(self):
        return self.__opts.indent


    @property
    def verbose(self):
        return self.__opts.verbose


    # ----------------------------------------------------------------------------------------------------------------

    def print_help(self, file):
        self.__parser.print_help(file)


    def __str__(self, *args, **kwargs):
        return "CmdDomainConf:{channel:%s, add:%s, remove:%s, indent:%s, verbose:%s}" % \
                    (self.channel, self.add, self.remove, self.indent, self.verbose)



"""
Created on 3 May 2021

@author: Bruno Beloff (bruno.beloff@southcoastscience.com)

https://packaging.python.org/guides/single-sourcing-package-version/
"""

__version__ = '3.3.11'


def version():
    return 'SCS Philips Hue ' + __version__



"""
Created on 27 Oct 2017

@author: Bruno Beloff (bruno.beloff@southcoastscience.com)

example:
"blah":
{"last use date": "2017-10-27T20:02:32", "create date": "2017-10-27T15:04:55", "name": "scs_hue#bruno"}
"""

from collections import OrderedDict

from scs_core.data.json import JSONable
from scs_core.data.str import Str

from scs_philips_hue.data.client.client_description import ClientDescription


# --------------------------------------------------------------------------------------------------------------------

class WhitelistGroup(JSONable):
    """
    classdocs
    """

    # ----------------------------------------------------------------------------------------------------------------

    @classmethod
    def construct_from_jdict(cls, jdict):
        if not jdict:
            return None

        entries = []

        for username, entry_jdict in jdict.items():
            entries.append(WhitelistEntry.construct_from_jdict(username, entry_jdict))

        return WhitelistGroup(entries)


    # ----------------------------------------------------------------------------------------------------------------

    def __init__(self, entries):
        """
        Constructor
        """
        self.__entries = entries                    # array of WhitelistEntry


    # ----------------------------------------------------------------------------------------------------------------

    def as_json(self):
        jdict = OrderedDict()

        for entry in self.entries:
            jdict[entry.username] = entry

        return jdict


    # ----------------------------------------------------------------------------------------------------------------

    @property
    def entries(self):
        return self.__entries


    # ----------------------------------------------------------------------------------------------------------------

    def __str__(self, *args, **kwargs):
        return "WhitelistGroup:{entries:%s}" % Str.collection(self.entries)


# --------------------------------------------------------------------------------------------------------------------

class WhitelistEntry(JSONable):
    """
    classdocs
    """

    # ----------------------------------------------------------------------------------------------------------------

    @classmethod
    def construct_from_jdict(cls, username, jdict):
        if not jdict:
            return None

        last_use_datetime = jdict.get('last use date')
        create_datetime = jdict.get('create date')

        description = ClientDescription.construct_from_jstr(jdict.get('name'))

        return WhitelistEntry(username, last_use_datetime, create_datetime, description)


    # ----------------------------------------------------------------------------------------------------------------

    def __init__(self, username, last_use_datetime, create_datetime, description):
        """
        Constructor
        """
        self.__username = username                                  # string

        self.__last_use_datetime = last_use_datetime                # string (date / time)
        self.__create_datetime = create_datetime                    # string (date / time)

        self.__description = description                            # ClientDescription


    # ----------------------------------------------------------------------------------------------------------------

    def as_json(self):
        jdict = OrderedDict()

        jdict['username'] = self.username
        jdict['last use date'] = self.last_use_datetime
        jdict['create date'] = self.create_datetime

        jdict['description'] = self.description

        return jdict


    # ----------------------------------------------------------------------------------------------------------------

    @property
    def username(self):
        return self.__username


    @property
    def last_use_datetime(self):
        return self.__last_use_datetime


    @property
    def create_datetime(self):
        return self.__create_datetime


    @property
    def description(self):
        return self.__description


    # ----------------------------------------------------------------------------------------------------------------

    def __str__(self, *args, **kwargs):
        return "WhitelistEntry:{username:%s, last_use_datetime:%s, create_datetime:%s, description:%s}" %  \
               (self.username, self.last_use_datetime, self.create_datetime, self.description)



"""
Created on 29 Oct 2017

@author: Bruno Beloff (bruno.beloff@southcoastscience.com)
"""

from scs_core.client.resource_unavailable_exception import ResourceUnavailableException

from scs_core.data.json import JSONable
from scs_core.data.str import Str

from scs_philips_hue.client.client_exception import ClientException
from scs_philips_hue.client.rest_client import RESTClient

from scs_philips_hue.data.bridge.bridge_config import ReportedBridgeConfig
from scs_philips_hue.data.bridge.response import Response

from scs_philips_hue.manager.manager import Manager


# --------------------------------------------------------------------------------------------------------------------

class BridgeManager(Manager):
    """
    classdocs
    """

    __TIMEOUT = 5               # seconds

    # ----------------------------------------------------------------------------------------------------------------

    @classmethod
    def is_bridge(cls, ip_address):
        rest_client = RESTClient()

        try:
            # request...
            rest_client.connect(ip_address, None, timeout=cls.__TIMEOUT)
            jdict = rest_client.get('/api')

            if jdict is None:
                return False

            # response...
            return Response.construct_from_jdict(jdict) is not None

        except (ClientException, ResourceUnavailableException):
            return False

        finally:
            rest_client.close()


    # ----------------------------------------------------------------------------------------------------------------

    def __init__(self, host, username):
        """
        Constructor
        """
        super().__init__(host, username)


    # ----------------------------------------------------------------------------------------------------------------

    def find(self):
        request_path = '/config'

        # request...
        self._rest_client.connect(self._host, self._username, timeout=self.__TIMEOUT)

        try:
            jdict = self._rest_client.get(request_path)
        finally:
            self._rest_client.close()

        # response...
        config = ReportedBridgeConfig.construct_from_jdict(jdict)

        return config


    def set_config(self, config):
        request_path = '/config'

        # request...
        self._rest_client.connect(self._host, self._username, timeout=self.__TIMEOUT)

        try:
            jdict = self._rest_client.put(request_path, config.as_json())
        finally:
            self._rest_client.close()

        # response...
        response = Response.construct_from_jdict(jdict)

        return response


    # ----------------------------------------------------------------------------------------------------------------

    def register(self, device):
        # request...
        self._rest_client.connect(self._host, None, timeout=self.__TIMEOUT)

        try:
            jdict = self._rest_client.post('', device.as_json())
        finally:
            self._rest_client.close()

        # response...
        response = Response.construct_from_jdict(jdict)

        return response


# --------------------------------------------------------------------------------------------------------------------

class BridgeManagerGroup(JSONable):
    """
    classdocs
    """

    # ----------------------------------------------------------------------------------------------------------------

    @classmethod
    def construct_from_jdict(cls, jdict):
        if not jdict:
            return None

        bridge_managers = {bridge_name: BridgeManager.construct_from_jdict(manager_jdict)
                           for bridge_name, manager_jdict in jdict}

        return cls(bridge_managers)


    # ----------------------------------------------------------------------------------------------------------------

    def __init__(self, bridge_managers):
        """
        Constructor
        """
        self.__bridge_managers = bridge_managers            # dict of bridge_name: BridgeManager


    def __bool__(self):
        return len(self.__bridge_managers) > 0


    # ----------------------------------------------------------------------------------------------------------------

    def is_complete(self):
        for manager in self.__bridge_managers.values():
            if manager is None:
                return False

        return True


    # ----------------------------------------------------------------------------------------------------------------

    def as_json(self):
        return self.__bridge_managers


    # ----------------------------------------------------------------------------------------------------------------

    def items(self):
        return dict(sorted(self.__bridge_managers.items())).items()


    # ----------------------------------------------------------------------------------------------------------------

    def __str__(self, *args, **kwargs):
        return "BridgeManagerGroup:{bridge_managers:%s}" % Str.collection(self.__bridge_managers)



"""
Created on 2 Apr 2018

@author: Bruno Beloff (bruno.beloff@southcoastscience.com)

example document:
{"org-id": "south-coast-science-test-user", "api-key": "9fdfb841-3433-45b8-b223-3f5a283ceb8e"}
"""

import optparse

from scs_philips_hue import version


# --------------------------------------------------------------------------------------------------------------------

class CmdAWSClientAuth(object):
    """unix command line handler"""

    def __init__(self):
        """
        Constructor
        """
        self.__parser = optparse.OptionParser(usage="%prog [{ [-e ENDPOINT] [-c CLIENT_ID] [-I CERT_ID] | -d }] [-v]",
                                              version=version())

        # fields...
        self.__parser.add_option("--endpoint", "-e", type="string", action="store", dest="endpoint",
                                 help="set broker endpoint")

        self.__parser.add_option("--client", "-c", type="string", action="store", dest="client_id",
                                 help="set client ID")

        self.__parser.add_option("--cert", "-i", type="string", action="store", dest="cert_id",
                                 help="set certificate ID")

        # delete...
        self.__parser.add_option("--delete", "-d", action="store_true", dest="delete", default=False,
                                 help="delete the client authentication")

        # output...
        self.__parser.add_option("--verbose", "-v", action="store_true", dest="verbose", default=False,
                                 help="report narrative to stderr")

        self.__opts, self.__args = self.__parser.parse_args()


    # ----------------------------------------------------------------------------------------------------------------

    def is_valid(self):
        if self.set() and self.delete:
            return False

        return True


    def is_complete(self):
        if self.endpoint is None or self.client_id is None or self.cert_id is None:
            return False

        return True


    def set(self):
        if self.endpoint is None and self.client_id is None and self.cert_id is None:
            return False

        return True


    # ----------------------------------------------------------------------------------------------------------------

    @property
    def endpoint(self):
        return self.__opts.endpoint


    @property
    def client_id(self):
        return self.__opts.client_id


    @property
    def cert_id(self):
        return self.__opts.cert_id


    @property
    def delete(self):
        return self.__opts.delete


    @property
    def verbose(self):
        return self.__opts.verbose


    # ----------------------------------------------------------------------------------------------------------------

    def print_help(self, file):
        self.__parser.print_help(file)


    def __str__(self, *args, **kwargs):
        return "CmdAWSClientAuth:{endpoint:%s, client_id:%s, cert_id:%s, delete:%s, verbose:%s}" % \
               (self.endpoint, self.client_id, self.cert_id, self.delete, self.verbose)



#!/usr/bin/env python3

"""
Created on 3 Nov 2017

@author: Bruno Beloff (bruno.beloff@southcoastscience.com)

source repo: scs_philips_hue

DESCRIPTION
The join utility is used to pair a controller device - the device hosting this software - with a Philips Hue Bridge.
Once running join, the big button on the top of the Philips Hue Bridge must be pressed!

The bridge_credentials.json document created by the join utility is used by the bridge, desk, light, and user utilities.

SYNOPSIS
join.py [-i INDENT] [-v] BRIDGE_NAME

EXAMPLES
./join.py -v

FILES
~/SCS/hue/bridge_address_set.json
~/SCS/hue/bridge_credentials_set.json

DOCUMENT EXAMPLE
{
    "hue-br1-001": {
        "bridge-id": "001788FFFEAF8430",
        "username": "DN5YzKBncC6n69gjlKZqa6SRqZofXTmhZkrdnqG2"
    },
    "hue-br1-002": {
        "bridge-id": "001788FFFE795620",
        "username": "suLJJv6OgxG0UjwB9Uz7e1j08Xj36MOfFdxNNUmR"
    }
}

SEE ALSO
scs_philips_hue/bridge
scs_philips_hue/desk
scs_philips_hue/light
scs_philips_hue/user
"""

import sys

from scs_core.client.http_exception import HTTPException
from scs_core.client.network import Network
from scs_core.client.resource_unavailable_exception import ResourceUnavailableException

from scs_core.data.json import JSONify

from scs_core.sys.logging import Logging

from scs_host.comms.stdio import StdIO
from scs_host.sys.host import Host

from scs_philips_hue.cmd.cmd_join import CmdJoin

from scs_philips_hue.config.bridge_address import BridgeAddress, BridgeAddressSet
from scs_philips_hue.config.bridge_credentials import BridgeCredentials, BridgeCredentialsSet

from scs_philips_hue.data.bridge.bridge_config import BridgeConfig

from scs_philips_hue.discovery.discovery import Discovery

from scs_philips_hue.manager.bridge_manager import BridgeManager
from scs_philips_hue.manager.user_manager import UserManager

from scs_philips_hue.data.client.client_description import ClientDescription
from scs_philips_hue.data.client.device_description import DeviceDescription


# --------------------------------------------------------------------------------------------------------------------

if __name__ == '__main__':

    bridge = None

    # ----------------------------------------------------------------------------------------------------------------
    # cmd...

    cmd = CmdJoin()

    if not cmd.is_valid():
        cmd.print_help(sys.stderr)
        exit(2)

    Logging.config('join', verbose=cmd.verbose)
    logger = Logging.getLogger()

    logger.info(cmd)

    try:
        # ------------------------------------------------------------------------------------------------------------
        # check...

        if not Network.is_available():
            logger.info("waiting for network")
            Network.wait()


        # ----------------------------------------------------------------------------------------------------------------
        # resources...

        # BridgeCredentialsSet...
        credentials_set = BridgeCredentialsSet.load(Host, skeleton=True)
        credentials_set.delete(cmd.bridge_name)
        credentials_set.save(Host)

        logger.info(credentials_set)

        # BridgeAddressSet...
        address_set = BridgeAddressSet.load(Host, skeleton=True)
        address_set.delete(cmd.bridge_name)
        address_set.save(Host)

        logger.info(address_set)

        # device...
        client = ClientDescription(ClientDescription.APP, Host.name())

        device = DeviceDescription(client)
        logger.info(device)

        # bridges...
        discovery = Discovery(Host)
        bridges = list(discovery.find_all())

        if len(bridges) == 0:
            logger.error("no bridge found.")
            exit(1)


        # ----------------------------------------------------------------------------------------------------------------
        # find bridge with button pressed...

        StdIO.prompt('Press the button! (Then hit RETURN)')

        success = None

        for bridge in bridges:
            logger.info(bridge)

            # manager...
            bridge_manager = BridgeManager(bridge.ip_address, None)

            # register...
            try:
                response = bridge_manager.register(device)
            except TimeoutError:
                logger.error("no response from bridge.")
                continue

            if response.has_errors():
                for error in response.errors:
                    logger.error("error: %s." % error.description)
                continue

            # get credentials...
            success = response.successes.pop()
            break

        if success is None:
            exit(1)


        # ----------------------------------------------------------------------------------------------------------------
        # run...

        # get bridge...
        bridge_manager = BridgeManager(bridge.ip_address, success.value)
        config = bridge_manager.find()

        # build credentials...
        credentials = BridgeCredentials(cmd.bridge_name, config.bridge_id, success.value)

        # delete old whitelist entries for this user (non-functional API)...
        user_manager = UserManager(bridge.ip_address, credentials.username)
        users = user_manager.find_all()

        for user in users:
            if user.description.user == Host.name() and user.username != credentials.username:
                response = user_manager.delete(user.username)

        # set bridge name...
        response = bridge_manager.set_config(BridgeConfig(name=cmd.bridge_name))

        # save address...
        address = BridgeAddress.construct(cmd.bridge_name, config.ip_address)
        address_set.add(address)
        address_set.save(Host)

        # save credentials...
        credentials_set.add(credentials)
        credentials_set.save(Host)

        # report...
        bridge_manager = BridgeManager(bridge.ip_address, credentials.username)
        config = bridge_manager.find()

        print(JSONify.dumps(config, indent=cmd.indent))


    # ----------------------------------------------------------------------------------------------------------------
    # end...

    except KeyboardInterrupt:
        print(file=sys.stderr)

    except (ConnectionError, HTTPException, ResourceUnavailableException) as ex:
        logger.error(repr(ex))
        exit(1)




"""
Created on 19 Nov 2017

@author: Bruno Beloff (bruno.beloff@southcoastscience.com)
"""

import optparse

from scs_core.data.datum import Datum

from scs_philips_hue import version
from scs_philips_hue.data.bridge.bridge_config import BridgeConfig


# --------------------------------------------------------------------------------------------------------------------

class CmdBridge(object):
    """unix command line handler"""

    def __init__(self):
        """
        Constructor
        """
        self.__parser = optparse.OptionParser(usage="%prog [-p PORTAL_SERVICES] [-c CHECK_UPDATE] [-u DO_UPDATE] "
                                                    "[-z CHANNEL] [-i INDENT] [-v] BRIDGE_NAME", version=version())

        # operations...
        self.__parser.add_option("--portal", "-p", type="int", action="store", dest="portal_services",
                                 help="enable portal services (1 or 0)")

        self.__parser.add_option("--check", "-c", type="int", action="store", dest="check_update",
                                 help="check for software update (1 or 0)")

        self.__parser.add_option("--update", "-u", type="int", action="store", dest="do_update",
                                 help="perform software update, if available (1 or 0)")

        self.__parser.add_option("--zigbee", "-z", type="string", action="store", dest="zigbee_channel",
                                 help="set zigbee channel (11, 15, 20 or 25)")

        # output...
        self.__parser.add_option("--indent", "-i", type="int", action="store", dest="indent",
                                 help="pretty-print the output with INDENT")

        self.__parser.add_option("--verbose", "-v", action="store_true", dest="verbose", default=False,
                                 help="report narrative to stderr")

        self.__opts, self.__args = self.__parser.parse_args()


    # ----------------------------------------------------------------------------------------------------------------

    def is_valid(self):
        if self.bridge_name is None:
            return False

        if self.__opts.check_update is not None:
            if self.__opts.check_update != 0 and self.__opts.check_update != 1:
                return False

        if self.__opts.do_update is not None:
            if self.__opts.do_update != 0 and self.__opts.do_update != 1:
                return False

        if self.zigbee_channel is not None:
            if self.__opts.zigbee not in BridgeConfig.ZIGBEE_CHANNELS:
                return False

        return True


    def set(self):
        return self.__opts.update is not None


    # ----------------------------------------------------------------------------------------------------------------

    @property
    def portal_services(self):
        return Datum.bool(self.__opts.portal_services)


    @property
    def check_update(self):
        return Datum.bool(self.__opts.check_update)


    @property
    def do_update(self):
        return Datum.bool(self.__opts.do_update)


    @property
    def zigbee_channel(self):
        return self.__opts.zigbee_channel


    @property
    def indent(self):
        return self.__opts.indent


    @property
    def verbose(self):
        return self.__opts.verbose


    @property
    def bridge_name(self):
        return self.__args[0] if self.__args else None


    # ----------------------------------------------------------------------------------------------------------------

    def print_help(self, file):
        self.__parser.print_help(file)


    def __str__(self, *args, **kwargs):
        return "CmdBridge:{bridge_name:%s, portal_services:%s, check_update:%s, do_update:%s, zigbee_channel:%s, " \
               "indent:%s, verbose:%s}" %  \
               (self.bridge_name, self.portal_services, self.check_update, self.do_update, self.zigbee_channel,
                self.indent, self.verbose)



"""
Created on 16 Mar 2018

@author: Bruno Beloff (bruno.beloff@southcoastscience.com)

document example:
{"NO2": {"path-channel": "risk_level", "domain-min": 0.0, "domain-max": 50.0, "brightness": 254, "transition-time": 9}}
"""

import optparse

from scs_philips_hue import version
from scs_philips_hue.config.chroma_path import ChromaPath


# --------------------------------------------------------------------------------------------------------------------

class CmdChromaConf(object):
    """unix command line handler"""

    def __init__(self):
        """
        Constructor
        """
        path_names = ' | '.join(ChromaPath.list())

        self.__parser = optparse.OptionParser(usage="%prog [-c CHANNEL { [-p PATH_NAME] [-l DOMAIN_MIN] "
                                                    "[-u DOMAIN_MAX] [-b BRIGHTNESS] [-t TRANSITION] | -r }] "
                                                    "[-i INDENT] [-v]",
                                              version=version())

        # configuration...
        self.__parser.add_option("--channel", "-c", type="string", action="store", dest="channel",
                                 help="the name of the information channel")

        # fields...
        self.__parser.add_option("--path", "-p", type="string", action="store", dest="path_name",
                                 help="channel of chroma path { %s }" % path_names)

        self.__parser.add_option("--lower", "-l", type="float", action="store", dest="domain_min",
                                 help="specify the domain lower bound")

        self.__parser.add_option("--upper", "-u", type="float", action="store", dest="domain_max",
                                 help="specify the domain upper bound")

        self.__parser.add_option("--bright", "-b", type="int", action="store", dest="brightness",
                                 help="set the lamp brightness (max 254)")

        self.__parser.add_option("--trans", "-t", type="float", action="store", dest="transition_time",
                                 help="set the lamp transition time (seconds)")

        # operations...
        self.__parser.add_option("--remove", "-r", action="store_true", dest="remove", default=False,
                                 help="remove the given configuration")

        # output...
        self.__parser.add_option("--indent", "-i", type="int", action="store", dest="indent",
                                 help="pretty-print the output with INDENT")

        self.__parser.add_option("--verbose", "-v", action="store_true", dest="verbose", default=False,
                                 help="report narrative to stderr")

        self.__opts, self.__args = self.__parser.parse_args()


    # ----------------------------------------------------------------------------------------------------------------

    def is_valid(self):
        if self.set() and self.channel is None:
            return False

        if self.set() and self.remove:
            return False

        if self.path_name is not None and self.path_name not in ChromaPath.list():
            return False

        if self.brightness is not None and (self.brightness < 0 or self.brightness > 254):
            return False

        return True


    def is_complete(self):
        if self.path_name is None or self.domain_min is None or self.domain_max is None or \
                self.brightness is None or self.transition_time is None:
            return False

        return True


    def set(self):
        return self.path_name is not None or self.domain_min is not None or self.domain_max is not None or \
               self.brightness is not None or self.transition_time is not None


    # ----------------------------------------------------------------------------------------------------------------

    @property
    def channel(self):
        return self.__opts.channel


    @property
    def path_name(self):
        return self.__opts.path_name


    @property
    def domain_min(self):
        return self.__opts.domain_min


    @property
    def domain_max(self):
        return self.__opts.domain_max


    @property
    def brightness(self):
        return self.__opts.brightness


    @property
    def transition_time(self):
        return self.__opts.transition_time


    @property
    def remove(self):
        return self.__opts.remove


    @property
    def indent(self):
        return self.__opts.indent


    @property
    def verbose(self):
        return self.__opts.verbose


    # ----------------------------------------------------------------------------------------------------------------

    def print_help(self, file):
        self.__parser.print_help(file)


    def __str__(self, *args, **kwargs):
        return "CmdChromaConf:{channel:%s, path_name:%s, domain_min:%s, domain_max:%s, " \
               "brightness:%s, transition_time:%s, remove:%s, indent:%s, verbose:%s}" % \
                    (self.channel, self.path_name, self.domain_min, self.domain_max,
                     self.brightness, self.transition_time, self.remove, self.indent, self.verbose)



"""
Created on 16 Mar 2018

@author: Bruno Beloff (bruno.beloff@southcoastscience.com)

document example:
[{"topic-path": "south-coast-science-demo/brighton/loc/1/particulates", "document-node": "exg.val.pm10"},
{"topic-path": "south-coast-science-demo/brighton/loc/1/particulates", "document-node": "exg.val.pm2p5"}]
"""

from collections import OrderedDict

from scs_core.data.json import JSONable

from scs_philips_hue.config.conf_set import ConfSet


# --------------------------------------------------------------------------------------------------------------------

class DomainConfSet(ConfSet):
    """
    classdocs
    """

    __FILENAME =        "domain_conf_set.json"

    @classmethod
    def persistence_location(cls):
        return cls.hue_dir(), cls.__FILENAME


    # ----------------------------------------------------------------------------------------------------------------

    @classmethod
    def construct_from_jdict(cls, jdict, skeleton=False):
        if not jdict:
            return cls({}) if skeleton else None

        confs = {}

        for name, conf_jdict in jdict.items():
            confs[name] = DomainConf.construct_from_jdict(conf_jdict)

        return cls(confs)


    # ----------------------------------------------------------------------------------------------------------------

    def __init__(self, confs):
        """
        Constructor
        """
        super().__init__(confs)


    # ----------------------------------------------------------------------------------------------------------------

    def add(self, name, topic_path, document_node):
        self._confs[name] = DomainConf(topic_path, document_node)


    def topic_paths(self):
        topic_path_set = set((conf.topic_path for conf in self._confs.values()))

        return topic_path_set


# --------------------------------------------------------------------------------------------------------------------

class DomainConf(JSONable):
    """
    classdocs
    """

    # ----------------------------------------------------------------------------------------------------------------

    @classmethod
    def construct_from_jdict(cls, jdict):
        if not jdict:
            return None

        topic_path = jdict.get('topic-path')
        document_node = jdict.get('document-node')

        return cls(topic_path, document_node)


    # ----------------------------------------------------------------------------------------------------------------

    def __init__(self, topic_path, document_node):
        """
        Constructor
        """
        self.__topic_path = topic_path                          # string
        self.__document_node = document_node                    # string


    def __lt__(self, other):
        if self.topic_path < other.topic_path:
            return True

        if self.topic_path > other.topic_path:
            return False

        if self.document_node < other.document_node:
            return True

        if self.document_node > other.document_node:
            return False

        return False


    # ----------------------------------------------------------------------------------------------------------------

    def as_json(self):
        jdict = OrderedDict()

        jdict['topic-path'] = self.topic_path
        jdict['document-node'] = self.document_node

        return jdict


    # ----------------------------------------------------------------------------------------------------------------

    @property
    def node_path(self):
        return '.'.join((self.topic_path, self.document_node))


    @property
    def topic_path(self):
        return self.__topic_path


    @property
    def document_node(self):
        return self.__document_node


    # ----------------------------------------------------------------------------------------------------------------

    def __str__(self, *args, **kwargs):
        return "DomainConf:{topic_path:%s, document_node:%s}" % (self.topic_path, self.document_node)



"""
Created on 28 Oct 2017

@author: Bruno Beloff (bruno.beloff@southcoastscience.com)

example:
{
    "on": true,
    "bri": 254,
    "hue": 8418,
    "sat": 140,
    "effect": "none",
    "xy": [0.4573, 0.41],
    "ct": 366,
    "alert": "select",
    "color_mode": "ct",
    "reachable": true
}
"""

from collections import OrderedDict

from scs_core.data.json import JSONable

from scs_philips_hue.data.light.chroma import ChromaPoint


# --------------------------------------------------------------------------------------------------------------------

class LightState(JSONable):
    """
    classdocs
    """

    ALERT_NONE =        'none'
    ALERT_SELECT =      'select'
    ALERT_L_SELECT =    'lselect'

    # ----------------------------------------------------------------------------------------------------------------

    @classmethod
    def white(cls, bri=254):
        return cls(on=True, bri=bri, hue=8418, sat=140, transition_time=1.0)


    # ----------------------------------------------------------------------------------------------------------------

    @classmethod
    def construct_from_jdict(cls, jdict):
        if not jdict:
            return None

        on = jdict.get('on')
        bri = jdict.get('bri')
        hue = jdict.get('hue')
        sat = jdict.get('sat')
        effect = jdict.get('effect')

        transition_time_tenths = jdict.get('transitiontime')
        transition_time = None if transition_time_tenths is None else round(transition_time_tenths / 10.0, 1)

        xy = ChromaPoint.construct_from_jdict(jdict.get('xy'))

        ct = jdict.get('ct')
        alert = jdict.get('alert')

        return cls(on=on, bri=bri, hue=hue, sat=sat, effect=effect, transition_time=transition_time, xy=xy, ct=ct,
                   alert=alert)


    # ----------------------------------------------------------------------------------------------------------------

    def __init__(self, on=None, bri=None, hue=None, sat=None, effect=None, transition_time=None,
                 xy=None, ct=None, alert=None):
        """
        Constructor
        """
        self.__on = on                                  # bool
        self.__bri = bri                                # uint8
        self.__hue = hue                                # uint16
        self.__sat = sat                                # uint8
        self.__effect = effect                          # string
        self.__transition_time = transition_time        # uint16 (seconds x 10)

        self.__xy = xy                                  # ChromaPoint

        self.__ct = ct                                  # uint16
        self.__alert = alert                            # string


    # ----------------------------------------------------------------------------------------------------------------

    def as_json(self):
        jdict = OrderedDict()

        if self.on is not None:
            jdict['on'] = self.on

        if self.bri is not None:
            jdict['bri'] = self.bri

        if self.hue is not None:
            jdict['hue'] = self.hue

        if self.sat is not None:
            jdict['sat'] = self.sat

        if self.effect is not None:
            jdict['effect'] = self.effect

        if self.transition_time is not None:
            jdict['transitiontime'] = int(self.transition_time * 10.0)

        if self.xy is not None:
            jdict['xy'] = self.xy

        if self.ct is not None:
            jdict['ct'] = self.ct

        if self.alert is not None:
            jdict['alert'] = self.alert

        return jdict


    # ----------------------------------------------------------------------------------------------------------------

    @property
    def on(self):
        return self.__on


    @property
    def bri(self):
        return self.__bri


    @property
    def hue(self):
        return self.__hue


    @property
    def sat(self):
        return self.__sat


    @property
    def effect(self):
        return self.__effect


    @property
    def transition_time(self):
        return self.__transition_time


    @property
    def xy(self):
        return self.__xy


    @property
    def ct(self):
        return self.__ct


    @property
    def alert(self):
        return self.__alert


    # ----------------------------------------------------------------------------------------------------------------

    def __str__(self, *args, **kwargs):
        return "LightState:{on:%s, bri:%s, hue:%s, sat:%s, effect:%s, transition_time:%s, " \
               "xy:%s, ct:%s, alert:%s}" %  \
               (self.on, self.bri, self.hue, self.sat, self.effect, self.transition_time,
                self.xy, self.ct, self.alert)


# --------------------------------------------------------------------------------------------------------------------

class ReportedLightState(LightState):
    """
    classdocs
    """

    # ----------------------------------------------------------------------------------------------------------------

    @classmethod
    def construct_from_jdict(cls, jdict):
        if not jdict:
            return None

        on = jdict.get('on')
        bri = jdict.get('bri')
        hue = jdict.get('hue')
        sat = jdict.get('sat')

        effect = jdict.get('effect')
        transition_time = jdict.get('transitiontime')

        xy = ChromaPoint.construct_from_jdict(jdict.get('xy'))

        ct = jdict.get('ct')
        alert = jdict.get('alert')

        color_mode = jdict.get('colormode')
        reachable = jdict.get('reachable')

        return ReportedLightState(on=on, bri=bri, hue=hue, sat=sat, effect=effect, transition_time=transition_time,
                                  xy=xy, ct=ct, alert=alert, color_mode=color_mode, reachable=reachable)


    # ----------------------------------------------------------------------------------------------------------------

    def __init__(self, on=None, bri=None, hue=None, sat=None, effect=None, transition_time=None,
                 xy=None, ct=None, alert=None, color_mode=None, reachable=None):
        """
        Constructor
        """
        super().__init__(on=on, bri=bri, hue=hue, sat=sat, effect=effect, transition_time=transition_time,
                         xy=xy, ct=ct, alert=alert)

        self.__color_mode = color_mode              # string
        self.__reachable = reachable                # bool


    # ----------------------------------------------------------------------------------------------------------------

    def as_json(self):
        jdict = OrderedDict()

        jdict['on'] = self.on
        jdict['bri'] = self.bri
        jdict['hue'] = self.hue
        jdict['sat'] = self.sat

        jdict['effect'] = self.effect
        jdict['transitiontime'] = self.transition_time

        jdict['xy'] = self.xy

        jdict['ct'] = self.ct
        jdict['alert'] = self.alert

        jdict['colormode'] = self.color_mode
        jdict['reachable'] = self.reachable

        return jdict


    # ----------------------------------------------------------------------------------------------------------------

    @property
    def color_mode(self):
        return self.__color_mode


    @property
    def reachable(self):
        return self.__reachable


    # ----------------------------------------------------------------------------------------------------------------

    def __str__(self, *args, **kwargs):
        return "ReportedLightState:{on:%s, bri:%s, hue:%s, sat:%s, effect:%s, transition_time:%s, " \
               "xy:%s, ct:%s, alert:%s, color_mode:%s, reachable:%s}" %  \
               (self.on, self.bri, self.hue, self.sat, self.effect, self.transition_time,
                self.xy, self.ct, self.alert, self.color_mode, self.reachable)



#!/usr/bin/env python3

"""
Created on 2 Apr 2018

@author: Bruno Beloff (bruno.beloff@southcoastscience.com)

source repo: scs_philips_hue

DESCRIPTION
The aws_api_auth utility is used to store or read the endpoint host name, client ID and client certificate ID
required by the South Coast Science / AWS messaging infrastructure.

SYNOPSIS
aws_client_auth.py [{ [-e ENDPOINT] [-c CLIENT_ID] [-I CERT_ID] | -d }] [-v]

EXAMPLES
./aws_client_auth.py -e asrft7e5j5ecz.iot.us-west-2.amazonaws.com -c bruno -i 9f08402232

FILES
~/SCS/aws/aws_client_auth.json

~/SCS/aws/certs/XXX-certificate.pem.crt
~/SCS/aws/certs/XXX-private.pem.key
~/SCS/aws/certs/XXX-public.pem.key
~/SCS/aws/certs/root-CA.crt

DOCUMENT EXAMPLE
{"endpoint": "asrft7e5j5ecz.iot.us-west-2.amazonaws.com", "client-id": "bruno", "cert-id": "9f08402232"}

SEE ALSO
scs_philips_hue/aws_mqtt_subscriber
"""

import sys

from scs_core.aws.client.client_auth import ClientAuth
from scs_core.data.json import JSONify
from scs_core.sys.logging import Logging

from scs_host.sys.host import Host

from scs_philips_hue.cmd.cmd_aws_client_auth import CmdAWSClientAuth


# --------------------------------------------------------------------------------------------------------------------

if __name__ == '__main__':

    # ----------------------------------------------------------------------------------------------------------------
    # cmd...

    cmd = CmdAWSClientAuth()

    if not cmd.is_valid():
        cmd.print_help(sys.stderr)
        exit(2)

    Logging.config('aws_client_auth', verbose=cmd.verbose)
    logger = Logging.getLogger()

    logger.info(cmd)


    # ----------------------------------------------------------------------------------------------------------------
    # resources...

    # ClientAuth...
    auth = ClientAuth.load(Host)


    # ----------------------------------------------------------------------------------------------------------------
    # run...

    if cmd.set():
        if auth is None and not cmd.is_complete():
            logger.error("No configuration is stored - you must therefore set all fields.")
            exit(2)

        endpoint = cmd.endpoint if cmd.endpoint else auth.endpoint
        client_id = cmd.client_id if cmd.client_id else auth.client_id
        cert_id = cmd.cert_id if cmd.cert_id else auth.cert_id

        auth = ClientAuth(endpoint, client_id, cert_id)
        auth.save(Host)

    if cmd.delete:
        auth.delete(Host)
        auth = None

    if auth:
        print(JSONify.dumps(auth))



#!/usr/bin/env python3

"""
Created on 25 Nov 2017

@author: Bruno Beloff (bruno.beloff@southcoastscience.com)

source repo: scs_philips_hue

DESCRIPTION
The desk utility (a lighting control desk) is used to drive the Philips Hue bridge device. Input data is received
from stdin, and is interpreted as a scs_philips_hue.data.light.LightState document.

If a bridge address has been stored, this is used to find the bridge. Otherwise a UPnP or IP scan is attempted.

In verbose mode, the desk utility provides a detailed report on the command outcome to stderr.

The desk utility requires the desk_conf.json document, specifying which light(s) should receive the LightState
command.

Note: this utility waits forever for a network connection and domain name server.

SYNOPSIS
desk.py [-n NAME] [-e] [-v]

EXAMPLES
./aws_mqtt_subscriber.py -vc | ./node.py -vc | ./chroma.py -v | ./desk.py -ve

FILES
~/SCS/hue/desk_conf.json

DOCUMENT EXAMPLE - INPUT
{"bri": 254, "transitiontime": 90, "xy": [0.3704, 0.5848]}

SEE ALSO
scs_philips_hue/chroma
scs_philips_hue/desk_conf
"""

import json
import sys
import time

from scs_core.client.network import Network

from scs_core.sys.logging import Logging
from scs_core.sys.signalled_exit import SignalledExit

from scs_host.sys.host import Host

from scs_philips_hue.cmd.cmd_desk import CmdDesk

from scs_philips_hue.config.bridge_credentials import BridgeCredentialsSet
from scs_philips_hue.config.desk_conf import DeskConfSet

from scs_philips_hue.data.light.light_catalogue import LightCatalogue
from scs_philips_hue.data.light.light_state import LightState

from scs_philips_hue.manager.bridge_monitor import BridgeMonitor
from scs_philips_hue.manager.light_manager import LightManager


# --------------------------------------------------------------------------------------------------------------------

def flush_stdin():
    prev = time.time()
    for _ in sys.stdin:
        now = time.time()
        if now - prev > 0.1:
            break

        prev = now


def process_channels():
    for channel in datum.keys():
        if channel not in desk_conf_set:
            logger.info("encountered an unsupported channel: %s" % channel)
            exit(1)

        state = LightState.construct_from_jdict(datum[channel])
        process_lights(channel, state)


def process_lights(channel, state):
    for light_name in desk_conf_set.conf(channel).lamp_names:
        try:
            light = light_catalogue.light(light_name)
            manager = light_managers[light.bridge_name]
        except KeyError:
            logger.error("%s: light unreachable: %s" % (channel, light_name))
            continue

        try:
            response = manager.set_state(light.index, state)                    # may raise exception
            logger.info("%s: %s" % (light.bridge_name, response))
        except TimeoutError:
            logger.error("%s: timed out: %s" % (channel, light_name))
            continue


def reset_lights():
    if light_catalogue is None or light_managers is None:
        return

    try:
        for light in light_catalogue.sorted_entries.values():
            manager = light_managers[light.bridge_name]
            response = manager.set_state(light.index, LightState.white())
            logger.info("%s: %s" % (light.bridge_name, response))

    except Exception as ex:
        logger.error(repr(ex))


# --------------------------------------------------------------------------------------------------------------------

if __name__ == '__main__':

    light_catalogue = None
    light_managers = None

    # ----------------------------------------------------------------------------------------------------------------
    # cmd...

    cmd = CmdDesk()

    Logging.config('desk', verbose=cmd.verbose)
    logger = Logging.getLogger()

    logger.info(cmd)

    try:
        # ------------------------------------------------------------------------------------------------------------
        # check...

        if not Network.is_available():
            logger.info("waiting for network")
            Network.wait()


        # ------------------------------------------------------------------------------------------------------------
        # resources...

        # DeskConf...
        desk_conf_set = DeskConfSet.load(Host, cmd.name)

        if desk_conf_set is None:
            logger.error("DeskConfSet not available.")
            exit(1)

        logger.info(desk_conf_set)

        # BridgeCredentials...
        credentials_set = BridgeCredentialsSet.load(Host, skeleton=True)

        if len(credentials_set) < 1:
            logger.error("BridgeCredentials not available.")
            exit(1)

        monitor = BridgeMonitor.construct(Host, credentials_set)
        logger.info(monitor)


        # ------------------------------------------------------------------------------------------------------------
        # initialise...

        # signal handler...
        SignalledExit.construct("desk", cmd.verbose)

        monitor.start()


        # ------------------------------------------------------------------------------------------------------------
        # run...

        while True:
            # Phase 1...
            while True:
                time.sleep(5)
                group = monitor.bridge_manager_group

                if group:                         # at least one of the bridge managers has been found
                    logger.info(group)
                    break

            # Phase 2...
            flush_stdin()

            for line in sys.stdin:
                if line is None:
                    break

                if cmd.echo:
                    print(line.strip())
                    sys.stdout.flush()

                try:
                    datum = json.loads(line)
                except ValueError:
                    continue

                # update bridge managers...
                light_managers = LightManager.construct_all(monitor.bridge_manager_group)
                light_catalogue = LightCatalogue.construct(light_managers)

                process_channels()


    # ----------------------------------------------------------------------------------------------------------------
    # end...

    except KeyboardInterrupt:
        print(file=sys.stderr)

    except SystemExit:
        pass

    finally:
        logger.info("finishing")
        reset_lights()



"""
Created on 26 Apr 2022

@author: Bruno Beloff (bruno.beloff@southcoastscience.com)

source repo: scs_philips_hue

example document:
{"org-id": "south-coast-science-test-user", "api-key": "9fdfb841-3433-45b8-b223-3f5a283ceb8e"}
"""

import optparse

from scs_philips_hue import version


# --------------------------------------------------------------------------------------------------------------------

class CmdBridgeAddress(object):
    """unix command line handler"""

    def __init__(self):
        """
        Constructor
        """
        self.__parser = optparse.OptionParser(usage="%prog { -l | -r BRIDGE_NAME } [-i INDENT] [-v]",
                                              version=version())

        # operations...
        self.__parser.add_option("--remove", "-r", type="str", action="store", dest="remove",
                                 help="remove the named bridge")

        # output...
        self.__parser.add_option("--indent", "-i", type="int", action="store", dest="indent",
                                 help="pretty-print the output with INDENT")

        self.__parser.add_option("--verbose", "-v", action="store_true", dest="verbose", default=False,
                                 help="report narrative to stderr")

        self.__opts, self.__args = self.__parser.parse_args()


    # ----------------------------------------------------------------------------------------------------------------

    @property
    def remove(self):
        return self.__opts.remove


    @property
    def indent(self):
        return self.__opts.indent


    @property
    def verbose(self):
        return self.__opts.verbose


    # ----------------------------------------------------------------------------------------------------------------

    def print_help(self, file):
        self.__parser.print_help(file)


    def __str__(self, *args, **kwargs):
        return "CmdBridgeAddress:{remove:%s, indent:%s, verbose:%s}" % \
               (self.remove, self.indent, self.verbose)



"""
Created on 29 Oct 2017

@author: Bruno Beloff (bruno.beloff@southcoastscience.com)

example document:
{
    "hue-br1-001": {
        "bridge-id": "001788FFFEAF8430",
        "username": "DN5YzKBncC6n69gjlKZqa6SRqZofXTmhZkrdnqG2"
    },
    "hue-br1-002": {
        "bridge-id": "001788FFFE795620",
        "username": "suLJJv6OgxG0UjwB9Uz7e1j08Xj36MOfFdxNNUmR"
    }
}
"""

from collections import OrderedDict

from scs_core.data.json import JSONable, PersistentJSONable
from scs_core.data.str import Str


# --------------------------------------------------------------------------------------------------------------------

class BridgeCredentials(JSONable):
    """
    classdocs
    """

    # ----------------------------------------------------------------------------------------------------------------

    @classmethod
    def construct_from_jdict(cls, bridge_name, jdict):
        if not jdict:
            return None

        bridge_id = jdict.get('bridge-id')
        username = jdict.get('username')

        return cls(bridge_name, bridge_id, username)


    # ----------------------------------------------------------------------------------------------------------------

    def __init__(self, bridge_name, bridge_id, username):
        """
        Constructor
        """
        super().__init__()

        self.__bridge_name = bridge_name                        # string
        self.__bridge_id = bridge_id                            # string
        self.__username = username                              # string


    # ----------------------------------------------------------------------------------------------------------------

    def as_json(self):
        jdict = OrderedDict()

        jdict['bridge-id'] = self.bridge_id
        jdict['username'] = self.username

        return jdict


    # ----------------------------------------------------------------------------------------------------------------

    @property
    def bridge_name(self):
        return self.__bridge_name


    @property
    def bridge_id(self):
        return self.__bridge_id


    @property
    def username(self):
        return self.__username


    # ----------------------------------------------------------------------------------------------------------------

    def __str__(self, *args, **kwargs):
        return "BridgeCredentials:{bridge_name:%s, bridge_id:%s, username:%s}" % \
            (self.bridge_name, self.bridge_id, self.username)


# --------------------------------------------------------------------------------------------------------------------

class BridgeCredentialsSet(PersistentJSONable):
    """
    classdocs
    """

    __FILENAME = "bridge_credentials_set.json"

    @classmethod
    def persistence_location(cls):
        return cls.hue_dir(), cls.__FILENAME


    # ----------------------------------------------------------------------------------------------------------------

    @classmethod
    def construct_from_jdict(cls, jdict, skeleton=False):
        if not jdict:
            return cls(OrderedDict()) if skeleton else None

        credentials = OrderedDict()
        for bridge_name, credentials_jdict in jdict.items():
            credentials[bridge_name] = BridgeCredentials.construct_from_jdict(bridge_name, credentials_jdict)

        return cls(credentials)


    # ----------------------------------------------------------------------------------------------------------------

    def __init__(self, credentials):
        """
        Constructor
        """
        super().__init__()

        self.__credentials = credentials                 # dict of bridge_name: BridgeCredentials


    def __len__(self):
        return len(self.__credentials)


    def __getitem__(self, item):
        return self.__credentials[item]                             # may raise KeyError


    # ----------------------------------------------------------------------------------------------------------------

    def add(self, credentials: BridgeCredentials):
        self.__credentials[credentials.bridge_name] = credentials


    def delete(self, bridge_name):
        try:
            del self.__credentials[bridge_name]
            return True

        except KeyError:
            return False


    def credentials(self, bridge_name):
        return self.__credentials[bridge_name]                          # may raise KeyError


    # ----------------------------------------------------------------------------------------------------------------

    def as_json(self):
        return self.sorted_credentials


    # ----------------------------------------------------------------------------------------------------------------

    @property
    def sorted_credentials(self):
        return OrderedDict(sorted(self.__credentials.items()))


    # ----------------------------------------------------------------------------------------------------------------

    def __str__(self, *args, **kwargs):
        return "BridgeCredentialsSet:{credentials:%s}" % Str.collection(self.sorted_credentials)



#!/usr/bin/env python3

"""
Created on 11 Apr 2017

@author: Bruno Beloff (bruno.beloff@southcoastscience.com)

source repo: scs_philips_hue

DESCRIPTION
The node utility is used to extract a node within a JSON document. Data is presented as a sequence of documents on
stdin, the node is passed to stdout. The extracted node may be a leaf node or an internal node.

The node path can be specified either on the command line, or by referencing the domain_conf.json document.
The node utility may be set to either ignore documents that do not contain the specified node, or to terminate if the
node is not present.

SYNOPSIS
node.py {-c | -t TOPIC_PATH } [-i] [-v]

EXAMPLES
./aws_mqtt_subscriber.py -vc | ./node.py -vc | ./chroma.py -v | ./desk.py -ve

FILES
~/SCS/hue/domain_conf.json

SEE ALSO
scs_philips_hue/aws_mqtt_subscriber
scs_philips_hue/domain_conf
"""

import sys

from scs_core.data.json import JSONify
from scs_core.data.path_dict import PathDict

from scs_core.sys.logging import Logging
from scs_core.sys.signalled_exit import SignalledExit

from scs_host.sys.host import Host

from scs_philips_hue.cmd.cmd_node import CmdNode
from scs_philips_hue.config.domain_conf import DomainConf, DomainConfSet


# --------------------------------------------------------------------------------------------------------------------

if __name__ == '__main__':

    # ----------------------------------------------------------------------------------------------------------------
    # cmd...

    cmd = CmdNode()

    if not cmd.is_valid():
        cmd.print_help(sys.stderr)
        exit(2)

    Logging.config('node', verbose=cmd.verbose)
    logger = Logging.getLogger()

    logger.info(cmd)

    try:
        # ------------------------------------------------------------------------------------------------------------
        # resources...

        # DomainConf...
        if cmd.use_domain_conf:
            domains = DomainConfSet.load(Host)

            if domains is None:
                logger.error("DomainConfSet not available.")
                exit(1)

        else:
            domains = DomainConfSet({cmd.topic_path[0]: DomainConf(cmd.topic_path[1], cmd.topic_path[2])})
            logger.info(domains)


        # ------------------------------------------------------------------------------------------------------------
        # run...

        # signal handler...
        SignalledExit.construct("node", cmd.verbose)

        node = None

        for line in sys.stdin:
            datum = PathDict.construct_from_jstr(line)

            if datum is None:
                continue

            for name, domain in domains.confs.items():
                if datum.has_sub_path(domain.topic_path):
                    print(JSONify.dumps({name: datum.node(domain.node_path)}))
                    sys.stdout.flush()


    # ----------------------------------------------------------------------------------------------------------------
    # end...

    except (KeyboardInterrupt, SystemExit):
        pass

    finally:
        logger.info("finishing")



"""
Created on 29 Oct 2017

@author: Bruno Beloff (bruno.beloff@southcoastscience.com)
"""

from scs_philips_hue.data.bridge.bridge_config import ReportedBridgeConfig
from scs_philips_hue.data.bridge.response import Response

from scs_philips_hue.manager.manager import Manager


# --------------------------------------------------------------------------------------------------------------------

class UserManager(Manager):
    """
    classdocs
    """

    # ----------------------------------------------------------------------------------------------------------------

    def __init__(self, host, username):
        """
        Constructor
        """
        super().__init__(host, username)


    # ----------------------------------------------------------------------------------------------------------------

    def find(self, username):
        entries = self.find_all()

        for entry in entries:
            if entry.username == username:
                return entry

        return None


    def find_all(self):
        request_path = '/config'

        # request...
        self._rest_client.connect(self._host, self._username)

        try:
            jdict = self._rest_client.get(request_path)
        finally:
            self._rest_client.close()

        # response...
        config = ReportedBridgeConfig.construct_from_jdict(jdict)
        whitelist = config.whitelist

        return whitelist.entries if whitelist else []


    # ----------------------------------------------------------------------------------------------------------------

    def delete(self, username):
        request_path = '/config/whitelist/' + username

        # request...
        self._rest_client.connect(self._host, self._username)

        try:
            jdict = self._rest_client.delete(request_path)
        finally:
            self._rest_client.close()

        # response...
        response = Response.construct_from_jdict(jdict)

        return response



#!/usr/bin/env python3

"""
Created on 4 Nov 2017

@author: Bruno Beloff (bruno.beloff@southcoastscience.com)

source repo: scs_philips_hue

DESCRIPTION
The light utility is used to shifter a Philips Hue light bulb with a Philips Hue bridge device, to update bulb
settings.

If a bridge address has been stored, this is used to find the bridge. Otherwise a UPnP or IP scan is attempted.

SYNOPSIS
light.py { -c | -l BRIDGE_NAME | -s BRIDGE_NAME | -a BRIDGE_NAME SERIAL_NUMBER | -n BRIDGE_NAME INDEX LIGHT_NAME |
-r BRIDGE_NAME INDEX } [-i INDENT] [-v]

EXAMPLES
./light.py -vi4 -n hue-br1-001 1 "r&d"

FILES
~/SCS/hue/bridge_address_set.json
~/SCS/hue/bridge_credentials_set.json

DOCUMENT EXAMPLE - OUTPUT
{"5": {"state": {"on": false, "bri": 0, "hue": 0, "sat": 0, "effect": "none", "transitiontime": null, "xy": [0.0, 0.0],
"ct": 0, "alert": "none", "colormode": "hs", "reachable": false}, "swupdate": {"state": "transferring",
"lastinstall": null}, "type": "Extended color light", "name": "scs-hcl-001", "modelid": "LCT015",
"manufacturername": "Philips", "uniqueid": "00:17:88:01:03:54:25:66-0b", "swversion": "1.19.0_r19755",
"swconfigid": "A724919D", "productid": "Philips-LCT015-1-A19ECLv5"}}

SEE ALSO
scs_philips_hue/bridge
scs_philips_hue/join

BUGS
The light search function is not always successful if lights have been previously registered on the bridge. If a search
does not find the light, then the light should be acquired with -a BRIDGE_NAME SERIAL_NUMBER.
"""

import sys
import time

from scs_core.client.http_exception import HTTPException
from scs_core.client.network import Network
from scs_core.client.resource_unavailable_exception import ResourceUnavailableException

from scs_core.data.json import JSONify

from scs_core.sys.logging import Logging

from scs_host.sys.host import Host

from scs_philips_hue.cmd.cmd_light import CmdLight

from scs_philips_hue.config.bridge_credentials import BridgeCredentialsSet

from scs_philips_hue.manager.bridge_builder import BridgeBuilder

from scs_philips_hue.data.light.light_catalogue import LightCatalogue
from scs_philips_hue.data.light.light_device import LightDevice

from scs_philips_hue.manager.light_manager import LightManager


# --------------------------------------------------------------------------------------------------------------------

if __name__ == '__main__':

    manager = None

    # ----------------------------------------------------------------------------------------------------------------
    # cmd...

    cmd = CmdLight()

    if not cmd.is_valid():
        cmd.print_help(sys.stderr)
        exit(2)

    Logging.config('light', verbose=cmd.verbose)
    logger = Logging.getLogger()

    logger.info(cmd)

    try:
        # ------------------------------------------------------------------------------------------------------------
        # check...

        if not Network.is_available():
            logger.info("waiting for network")
            Network.wait()


        # ------------------------------------------------------------------------------------------------------------
        # resources...

        # BridgeCredentials...
        credentials_set = BridgeCredentialsSet.load(Host, skeleton=True)

        if len(credentials_set) < 1:
            logger.error("BridgeCredentials not available.")
            exit(1)

        # Managers...
        builder = BridgeBuilder(Host)

        bridge_managers = builder.construct_dict_for_credentials(credentials_set[cmd.bridge_name]) if cmd.bridge_name \
            else builder.construct_all(credentials_set)

        light_managers = LightManager.construct_all(bridge_managers)

        # LightCatalogue...
        light_catalogue = LightCatalogue.construct(light_managers)

        try:
            if cmd.bridge_name:
                manager = light_managers[cmd.bridge_name]
        except KeyError:
            logger.error("bridge '%s' was not found." % cmd.bridge_name)
            exit(1)


        # ------------------------------------------------------------------------------------------------------------
        # run...

        if cmd.name and cmd.light_name in light_catalogue:
            logger.error("name '%s' is already in use." % cmd.light_name)
            exit(2)


        # ------------------------------------------------------------------------------------------------------------
        # run...

        # search...
        if cmd.search:
            response = manager.search()
            logger.info("searching...")

            while True:
                time.sleep(2)
                scan = manager.find_new()
                if not scan.is_active():
                    break

            for entry in scan.entries:
                print(JSONify.dumps(entry, indent=cmd.indent))

        # add...
        if cmd.add:
            device = LightDevice(cmd.serial_number)
            response = manager.search(device=device)
            logger.info("adding...")

            while True:
                time.sleep(2.0)
                scan = manager.find_new()
                if not scan.is_active():
                    break

            for entry in scan.entries:
                print(JSONify.dumps(entry, indent=cmd.indent))

        # name...
        if cmd.name:
            response = manager.rename(cmd.index, cmd.light_name)

        # delete...
        if cmd.remove:
            response = manager.delete(cmd.index)

        # result...
        if cmd.catalogue:
            print(JSONify.dumps(light_catalogue, indent=cmd.indent))

        if cmd.name or cmd.remove or cmd.list:
            lights = manager.find_all()

            for light in lights:
                print(JSONify.dumps(light, indent=cmd.indent))


    # ----------------------------------------------------------------------------------------------------------------
    # end...

    except KeyboardInterrupt:
        print(file=sys.stderr)

    except (ConnectionError, HTTPException, ResourceUnavailableException) as ex:
        logger.error(repr(ex))
        exit(1)

    except TimeoutError:
        logger.error("Timeout")
        exit(1)



"""
Created on 27 Sep 2018

@author: Bruno Beloff (bruno.beloff@southcoastscience.com)
"""

import json

from scs_core.data.json import JSONify
from scs_core.data.publication import Publication

from scs_core.sys.logging import Logging


# --------------------------------------------------------------------------------------------------------------------
# subscription handler...

class AWSMQTTSubscriptionHandler(object):
    """
    classdocs
    """

    # ----------------------------------------------------------------------------------------------------------------

    def __init__(self, comms=None):
        """
        Constructor
        """
        self.__comms = comms                            # UDSWriter
        self.__logger = Logging.getLogger()


    # ----------------------------------------------------------------------------------------------------------------

    # noinspection PyUnusedLocal

    def handle(self, client, userdata, message):
        payload = message.payload.decode()
        payload_jdict = json.loads(payload)

        publication = Publication(message.topic, payload_jdict)

        try:
            self.__comms.connect()
            self.__comms.write(JSONify.dumps(publication), False)

        except ConnectionError:
            self.__logger.info("ConnectionError: %s" % self.__comms)

        finally:
            self.__comms.close()

        self.__logger.info(JSONify.dumps(publication))


    # ----------------------------------------------------------------------------------------------------------------

    def __str__(self, *args, **kwargs):
        return "AWSMQTTSubscriptionHandler:{comms:%s}" % self.__comms





#!/usr/bin/env python3

"""
Created on 16 Mar 2018

@author: Bruno Beloff (bruno.beloff@southcoastscience.com)

source repo: scs_philips_hue

DESCRIPTION
The desk_conf utility is used to specify which lamps should be driven by the desk utility.

SYNOPSIS
Usage: desk_conf.py [-c CHANNEL { -a LAMP_NAME | -r LAMP_NAME | -d }] [-i INDENT] [-v]

EXAMPLES
./desk_conf.py -vi4 -c assembly-co2 -a assembly

FILES
~/SCS/hue/desk_conf_set.json

DOCUMENT EXAMPLE
{
    "preston_circus": [
        "black"
    ]
}

SEE ALSO
scs_philips_hue/desk
"""

import sys

from scs_core.data.json import JSONify
from scs_core.sys.logging import Logging

from scs_host.sys.host import Host

from scs_philips_hue.cmd.cmd_desk_conf import CmdDeskConf
from scs_philips_hue.config.desk_conf import DeskConf, DeskConfSet


# --------------------------------------------------------------------------------------------------------------------

if __name__ == '__main__':

    desk = None

    # ----------------------------------------------------------------------------------------------------------------
    # cmd...

    cmd = CmdDeskConf()

    if not cmd.is_valid():
        cmd.print_help(sys.stderr)
        exit(2)

    Logging.config('desk_conf', verbose=cmd.verbose)
    logger = Logging.getLogger()

    logger.info(cmd)


    # ----------------------------------------------------------------------------------------------------------------
    # resources...

    # DeskConf...
    desks = DeskConfSet.load(Host, skeleton=True)


    # ----------------------------------------------------------------------------------------------------------------
    # run...

    if cmd.set():
        desk = desks.conf(cmd.channel)

    if cmd.add_lamp:
        if desk is None:
            desk = DeskConf([])

        desk.add_lamp(cmd.add_lamp)

        desks.add(cmd.channel, desk)
        desks.save(Host)

    if cmd.remove_lamp:
        if desk is not None:
            desk.remove_lamp(cmd.remove_lamp)

            if len(desk) == 0:
                desks.remove(cmd.channel)

            desks.save(Host)

    if cmd.delete:
        desks.remove(cmd.channel)
        desks.save(Host)

    if desks:
        print(JSONify.dumps(desks, indent=cmd.indent))



#!/usr/bin/env python3

"""
Created on 4 Oct 2017

@author: Bruno Beloff (bruno.beloff@southcoastscience.com)

source repo: scs_philips_hue

DESCRIPTION
The aws_mqtt_subscriber utility is used to obtain live data from an Amazon Web Services (AWS) messaging topic. The topic
path can be specified either on the command line, or by referencing the domain_conf.json document.

The aws_mqtt_subscriber passes subscribed data to stdout. Data is wrapped in a JSON document that uses the topic
path as a field name, thus identifying which topic the data was gained from.

In order to operate, the appropriate AWS certificates must be installed in a ~/SCS/aws/certs directory, and the
certificate and API auth specified in the client_credentials.json documents.

WARNING: only one MQTT client should run at any one time, per TCP/IP host.

SYNOPSIS
aws_mqtt_subscriber.py { -c | -t TOPIC_PATH } [-s UDS_SUB] [-v]

EXAMPLES
./aws_mqtt_subscriber.py -c | ./node.py -c | ./chroma.py | ./desk.py -v

FILES
~/SCS/aws/aws_client_auth.json
~/SCS/hue/domain_conf.json

SEE ALSO
scs_philips_hue/aws_client_auth
scs_philips_hue/aws_mqtt_subscriber
scs_philips_hue/domain_conf

BUGS
If the host is multi-homed and a higher-priority connection is lost, the MQTT connection will
not be recovered.
"""

import sys

from scs_core.aws.client.client_auth import ClientAuth
from scs_core.aws.client.mqtt_client import MQTTClient, MQTTClientManager, MQTTSubscriber

from scs_core.client.network import NetworkMonitor

from scs_core.comms.mqtt_conf import MQTTConf
from scs_core.comms.uds_writer import UDSWriter

from scs_core.sys.filesystem import Filesystem
from scs_core.sys.logging import Logging
from scs_core.sys.signalled_exit import SignalledExit

from scs_host.comms.domain_socket import DomainSocket
from scs_host.sys.host import Host

from scs_philips_hue.cmd.cmd_mqtt_subscriber import CmdMQTTSubscriber

from scs_philips_hue.config.domain_conf import DomainConfSet

from scs_philips_hue.handler.aws_mqtt_subscription_handler import AWSMQTTSubscriptionHandler


# --------------------------------------------------------------------------------------------------------------------

if __name__ == '__main__':

    conf = None
    domains = None
    publisher = None
    sub_comms = None

    # ----------------------------------------------------------------------------------------------------------------
    # cmd...

    cmd = CmdMQTTSubscriber()

    if not cmd.is_valid():
        cmd.print_help(sys.stderr)
        exit(2)

    Logging.config('aws_mqtt_subscriber', verbose=cmd.verbose)
    logger = Logging.getLogger()

    logger.info(cmd)

    try:
        # ------------------------------------------------------------------------------------------------------------
        # resources...

        # MQTTConf
        conf = MQTTConf.load(Host, skeleton=True)
        logger.info(conf)

        # ClientAuth...
        auth = ClientAuth.load(Host)

        if auth is None:
            logger.error("ClientAuth not available.")
            exit(1)

        # DomainConf...
        if cmd.use_domain_conf:
            domains = DomainConfSet.load(Host)

            if domains is None:
                logger.error("DomainConfSet not available.")
                exit(1)

            logger.info(domains)

        else:
            topic_path = cmd.topic_path

        # writer...
        sub_comms = UDSWriter(DomainSocket, cmd.uds_sub)

        # subscriber...
        handler = AWSMQTTSubscriptionHandler(comms=sub_comms)

        subscribers = []
        if cmd.use_domain_conf:
            for topic_path in domains.topic_paths():
                subscribers.append(MQTTSubscriber(topic_path, handler.handle))
        else:
            subscribers.append(MQTTSubscriber(cmd.topic_path, handler.handle))

        # client...
        MQTTClientManager.configure(auth, MQTTClient(*subscribers))

        # monitor...
        monitor = NetworkMonitor(10, MQTTClientManager.network_not_available_handler)
        logger.info(monitor)


        # ------------------------------------------------------------------------------------------------------------
        # run...

        # signal handler...
        SignalledExit.construct("aws_mqtt_subscriber", cmd.verbose)

        # connect...
        MQTTClientManager.connect()

        # monitor...
        monitor.start()
        monitor.join()


    # ----------------------------------------------------------------------------------------------------------------
    # end...

    except ConnectionError as ex:
        logger.error(repr(ex))

    except (KeyboardInterrupt, SystemExit):
        pass

    finally:
        logger.info("finishing")

        MQTTClientManager.disconnect()

        if sub_comms:
            sub_comms.close()

        if conf:
            Filesystem.rm(conf.report_file)



"""
Created on 25 Nov 2017

@author: Bruno Beloff (bruno.beloff@southcoastscience.com)
"""

import optparse

from scs_philips_hue import version


# --------------------------------------------------------------------------------------------------------------------

class CmdDesk(object):
    """unix command line handler"""

    def __init__(self):
        """
        Constructor
        """
        self.__parser = optparse.OptionParser(usage="%prog [-n NAME] [-e] [-v]", version=version())

        # identity...
        self.__parser.add_option("--name", "-n", type="string", action="store", dest="name",
                                 help="the name of the configuration")

        # output...
        self.__parser.add_option("--echo", "-e", action="store_true", dest="echo", default=False,
                                 help="echo stdin to stdout")

        self.__parser.add_option("--verbose", "-v", action="store_true", dest="verbose", default=False,
                                 help="report narrative to stderr")

        self.__opts, self.__args = self.__parser.parse_args()


    # ----------------------------------------------------------------------------------------------------------------

    @property
    def name(self):
        return self.__opts.name


    @property
    def echo(self):
        return self.__opts.echo


    @property
    def verbose(self):
        return self.__opts.verbose


    # ----------------------------------------------------------------------------------------------------------------

    def print_help(self, file):
        self.__parser.print_help(file)


    def __str__(self, *args, **kwargs):
        return "CmdDesk:{name:%s, echo:%s, verbose:%s}" %  (self.name, self.echo, self.verbose)



"""
Created on 4 Mar 2020

@author: Bruno Beloff (bruno.beloff@southcoastscience.com)
"""

from scs_core.sys.logging import Logging

from scs_philips_hue.discovery.upnp_conf import UPnPConf

from scs_philips_hue.discovery.ip_discovery import IPDiscovery
from scs_philips_hue.discovery.upnp_discovery import UPnPDiscovery


# --------------------------------------------------------------------------------------------------------------------

class Discovery(object):
    """
    classdocs
    """

    # ----------------------------------------------------------------------------------------------------------------

    def __init__(self, host):
        """
        Constructor
        """
        self.__host = host                                          # PersistenceManager

        self.__conf = UPnPConf.load(host, skeleton=True)
        self.__logger = Logging.getLogger()


    # ----------------------------------------------------------------------------------------------------------------

    def find(self, credentials):
        if self.__conf.upnp_enabled:
            # UPnP...
            self.__logger.info("find by UPnP...")
            upnp = UPnPDiscovery()
            bridge = upnp.find(credentials.bridge_id)

            if bridge:
                return bridge

        # IP scan...
        self.__logger.info("find by IP scan...")
        scanner = IPDiscovery(self.__host)

        return scanner.find(credentials)


    def find_all(self):
        if self.__conf.upnp_enabled:
            # UPnP...
            self.__logger.info("find by UPnP...")
            upnp = UPnPDiscovery()
            bridges = upnp.find_all()

            if bridges:
                return bridges

        # IP scan...
        self.__logger.info("find by IP scan...")
        scanner = IPDiscovery(self.__host)

        return scanner.find_all()


    # ----------------------------------------------------------------------------------------------------------------

    def __str__(self, *args, **kwargs):
        return "Discovery:{conf:%s}" % self.__conf



"""
Created on 4 Nov 2017

@author: Bruno Beloff (bruno.beloff@southcoastscience.com)
"""

import optparse

from scs_philips_hue import version


# --------------------------------------------------------------------------------------------------------------------

class CmdLight(object):
    """unix command line handler"""

    def __init__(self):
        """
        Constructor
        """
        self.__parser = optparse.OptionParser(usage="%prog { -c | -l BRIDGE_NAME | -s BRIDGE_NAME | "
                                                    "-a BRIDGE_NAME SERIAL_NUMBER | -n BRIDGE_NAME INDEX LIGHT_NAME | "
                                                    "-r BRIDGE_NAME INDEX } [-i INDENT] [-v]",
                                              version=version())

        # operations...
        self.__parser.add_option("--catalogue", "-c", action="store_true", dest="catalogue",
                                 help="catalogue of all light names")

        self.__parser.add_option("--list", "-l", type="string", action="store", dest="list",
                                 help="list lights attached to BRIDGE")

        self.__parser.add_option("--search", "-s", type="string", action="store", dest="search",
                                 help="search for new lights using BRIDGE")

        self.__parser.add_option("--add", "-a", type="string", nargs=2, action="store", dest="add",
                                 help="add the light with SERIAL_NUMBER to BRIDGE")

        self.__parser.add_option("--name", "-n", type="string", nargs=3, action="store", dest="name",
                                 help="set the name of the light with INDEX to NAME on BRIDGE")

        self.__parser.add_option("--remove", "-r", type="string", nargs=2, action="store", dest="remove",
                                 help="delete the light with INDEX from BRIDGE")

        # output...
        self.__parser.add_option("--indent", "-i", type="int", action="store", dest="indent",
                                 help="pretty-print the output with INDENT")

        self.__parser.add_option("--verbose", "-v", action="store_true", dest="verbose", default=False,
                                 help="report narrative to stderr")

        self.__opts, self.__args = self.__parser.parse_args()


    # ----------------------------------------------------------------------------------------------------------------

    def is_valid(self):
        count = 0

        if self.catalogue:
            count += 1

        if self.list:
            count += 1

        if self.search:
            count += 1

        if self.add:
            count += 1

        if self.name:
            count += 1

        if self.remove:
            count += 1

        if count != 1:
            return False

        try:
            _ = self.index
        except ValueError:
            return False

        return True


    # ----------------------------------------------------------------------------------------------------------------

    @property
    def catalogue(self):
        return self.__opts.catalogue


    @property
    def list(self):
        return self.__opts.list


    @property
    def search(self):
        return self.__opts.search is not None


    @property
    def add(self):
        return self.__opts.add is not None


    @property
    def name(self):
        return self.__opts.name is not None


    @property
    def remove(self):
        return self.__opts.remove is not None


    # ----------------------------------------------------------------------------------------------------------------

    @property
    def bridge_name(self):
        if self.list:
            return self.__opts.list

        if self.search:
            return self.__opts.search

        if self.add:
            return self.__opts.add[0]

        if self.name:
            return self.__opts.name[0]

        if self.remove:
            return self.__opts.remove[0]

        return None


    @property
    def serial_number(self):
        if self.add:
            return self.__opts.add[1]

        return None


    @property
    def index(self):
        if self.name:
            return int(self.__opts.name[1])

        if self.remove:
            return int(self.__opts.remove[1])

        return None


    @property
    def light_name(self):
        if self.name:
            return self.__opts.name[2]

        return None


    # ----------------------------------------------------------------------------------------------------------------

    @property
    def indent(self):
        return self.__opts.indent


    @property
    def verbose(self):
        return self.__opts.verbose


    # ----------------------------------------------------------------------------------------------------------------

    def print_help(self, file):
        self.__parser.print_help(file)


    def __str__(self, *args, **kwargs):
        return "CmdLight:{catalogue:%s, list:%s, search:%s, add:%s, name:%s, " \
               "remove:%s, indent:%s, verbose:%s}" %  \
               (self.__opts.catalogue, self.__opts.list, self.__opts.search, self.__opts.add, self.__opts.name,
                self.__opts.remove, self.indent, self.verbose)



#!/usr/bin/env python3

"""
Created on 2 Apr 2018

@author: Bruno Beloff (bruno.beloff@southcoastscience.com)

source repo: scs_analysis

DESCRIPTION
If the Philips Hue bridge has a known, fixed IP address and / or an IP scan is impractical, the bridge_address utility
may be used to record its IPv4 address. (The address can usually be found with a ZeroConf browser.)

SYNOPSIS
bridge_address.py { -l | -r BRIDGE_NAME } [-i INDENT] [-v]

EXAMPLES
./bridge_address.py -s 192.168.2.29

FILES
~/SCS/hue/bridge_address_set.json
~/SCS/hue/bridge_credentials_set.json

DOCUMENT EXAMPLE
{
    "hue-br1-001": {
        "ipv4": "192.168.1.16"
    },
    "hue-br1-002": {
        "ipv4": "192.168.1.8"
    }
}

SEE ALSO
scs_philips_hue/bridge
"""

from scs_core.data.json import JSONify
from scs_core.sys.logging import Logging

from scs_host.sys.host import Host

from scs_philips_hue.cmd.cmd_bridge_address import CmdBridgeAddress
from scs_philips_hue.config.bridge_address import BridgeAddressSet


# --------------------------------------------------------------------------------------------------------------------

if __name__ == '__main__':

    # ----------------------------------------------------------------------------------------------------------------
    # cmd...

    cmd = CmdBridgeAddress()

    Logging.config('bridge_address', verbose=cmd.verbose)
    logger = Logging.getLogger()

    logger.info(cmd)


    # ----------------------------------------------------------------------------------------------------------------
    # resources...

    # BridgeAddressSet...
    address_set = BridgeAddressSet.load(Host, skeleton=True)


    # ----------------------------------------------------------------------------------------------------------------
    # run...

    if cmd.remove:
        address_set.delete(cmd.remove)
        address_set.save(Host)

    print(JSONify.dumps(address_set, indent=cmd.indent))



"""
Created on 4 Nov 2017

@author: Bruno Beloff (bruno.beloff@southcoastscience.com)
"""

import optparse

from scs_philips_hue import version


# --------------------------------------------------------------------------------------------------------------------

class CmdUPnPConf(object):
    """unix command line handler"""

    def __init__(self):
        """
        Constructor
        """
        self.__parser = optparse.OptionParser(usage="%prog [-e { 0 | 1 }] [-i INDENT] [-v]", version=version())

        # functionality
        self.__parser.add_option("--enable", "-e", type="int", action="store", dest="enable",
                                 help="enable UPnP discovery")

        # output...
        self.__parser.add_option("--indent", "-i", type="int", action="store", dest="indent",
                                 help="pretty-print the output with INDENT")

        self.__parser.add_option("--verbose", "-v", action="store_true", dest="verbose", default=False,
                                 help="report narrative to stderr")

        self.__opts, self.__args = self.__parser.parse_args()


    # ----------------------------------------------------------------------------------------------------------------

    def is_valid(self):
        if self.__opts.enable is not None and self.__opts.enable not in [0, 1]:
            return False

        if self.__args:
            return False

        return True


    def set(self):
        return self.__opts.enable is not None


    # ----------------------------------------------------------------------------------------------------------------

    @property
    def enable(self):
        return None if self.__opts.enable is None else bool(self.__opts.enable)


    @property
    def indent(self):
        return self.__opts.indent


    @property
    def verbose(self):
        return self.__opts.verbose


    # ----------------------------------------------------------------------------------------------------------------

    def print_help(self, file):
        self.__parser.print_help(file)


    def __str__(self, *args, **kwargs):
        return "CmdUPnPConf:{enable:%s, indent:%s, verbose:%s}" % \
            (self.__opts.enable, self.indent, self.verbose)



"""
Created on 9 Nov 2023

@author: Bruno Beloff (bruno.beloff@southcoastscience.com)

https://packaging.python.org/tutorials/packaging-projects/
https://packaging.python.org/guides/single-sourcing-package-version/
"""

import codecs
import os
import setuptools


# --------------------------------------------------------------------------------------------------------------------

def read(rel_path):
    here = os.path.abspath(os.path.dirname(__file__))
    with codecs.open(os.path.join(here, rel_path)) as fp:
        return fp.read()


def get_version(rel_path):
    for line in read(rel_path).splitlines():
        if line.startswith('__version__'):
            return line.split("'")[1]
    else:
        raise RuntimeError("Unable to find version string.")


# --------------------------------------------------------------------------------------------------------------------

with open("README.md", "r") as fh:
    long_description = fh.read()

with open('requirements.txt') as req_txt:
    required = [line for line in req_txt.read().splitlines() if line]


setuptools.setup(
    name="scs-hue",
    version=get_version("src/scs_philips_hue/__init__.py"),
    author="South Coast Science",
    author_email="contact@southcoastscience.com",
    description="Connecting Philips Hue light bulbs to South Coast Science environmental data sources",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/south-coast-science/scs_philips_hue",
    package_dir={'': 'src'},
    packages=setuptools.find_packages('src'),
    package_data={'src': ['**/*.csv', '**/*.json']},
    include_package_data=True,
    classifiers=[
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.11",
        "License :: OSI Approved :: MIT License",
        "Operating System :: MacOS :: MacOS X",
        "Operating System :: Microsoft :: Windows :: Windows 10",
        "Operating System :: POSIX",
    ],
    scripts=[
        'src/scs_philips_hue/aws_client_auth.py',
        'src/scs_philips_hue/aws_mqtt_subscriber.py',
        'src/scs_philips_hue/bridge.py',
        'src/scs_philips_hue/bridge_address.py',
        'src/scs_philips_hue/chroma.py',
        'src/scs_philips_hue/chroma_conf.py',
        'src/scs_philips_hue/desk.py',
        'src/scs_philips_hue/desk_conf.py',
        'src/scs_philips_hue/domain_conf.py',
        'src/scs_philips_hue/join.py',
        'src/scs_philips_hue/light.py',
        'src/scs_philips_hue/node.py',
        'src/scs_philips_hue/run_chroma.sh',
        'src/scs_philips_hue/uds_receiver.py',
        'src/scs_philips_hue/upnp_conf.py',
        'src/scs_philips_hue/user.py'
    ],
    install_requires=required,
    platforms=['any'],
    python_requires='>3.6',
    extras_require={
        'dev': [
            'pypandoc'
        ]
    }
)



"""
Created on 28 Oct 2017

@author: Bruno Beloff (bruno.beloff@southcoastscience.com)

example:
{
    "name": "scs-hb-001",
    "zigbeechannel": 20,
    "bridgeid": "001788FFFE795620",
    "mac": "00:17:88:79:56:20",
    "dhcp": true,
    "ipaddress": "192.168.1.10",
    "netmask": "255.255.255.0",
    "gateway": "192.168.1.1",
    "proxyaddress": "none",
    "proxyport": 0,
    "UTC": "2017-10-28T14:16:10",
    "localtime": "2017-10-28T15:16:10",
    "timezone": "Europe/London",
    "modelid": "BSB002",
    "datastoreversion": "63",
    "swversion": "1709131301",
    "apiversion": "1.21.0",
    "swupdate": {
        "updatestate": 0,
        "checkforupdate": false,
        "devicetypes": {
            "bridge": false,
            "lights": [],
            "sensors": []
        },
        "url": "",
        "text": "",
        "notify": false
    },
    "swupdate2": {
        "checkforupdate": false,
        "lastchange": "2017-10-27T15:10:38",
        "bridge": {
            "state": "noupdates",
            "lastinstall": null
        },
        "state": "noupdates",
        "autoinstall": {
            "updatetime": "T14:00:00",
            "on": false
        }
    },
    "linkbutton": false,
    "portalservices": true,
    "portalconnection": "connected",
    "portalstate": {
        "signedon": true,
        "incoming": false,
        "outgoing": true,
        "communication": "disconnected"
    },
    "internetservices": {
        "internet": "connected",
        "remoteaccess": "connected",
        "time": "connected",
        "swupdate": "connected"
    },
    "factorynew": false,
    "replacesbridgeid": null,
    "backup": {
        "status": "idle",
        "errorcode": 0
    },
    "starterkitid": "",
    "whitelist": {
        "kakoh96DKsic6XGC9-v07nVIUig1naMOgv849i1r": {
            "last use date": "2017-10-27T20:02:32",
            "create date": "2017-10-27T15:04:55",
            "name": "scs_hue#bruno"
        },
        "11uK-QDNOytTk7UW6smCBYBVZXxKaFy3b72b7Qdv": {
            "last use date": "2017-10-27T15:17:32",
            "create date": "2017-10-27T15:10:10",
            "name": "Hue 2#HTC U11"
        }
    }
}
"""

from collections import OrderedDict

# from scs_core.data.json import JSONify

from scs_core.data.datum import Datum
from scs_core.data.json import JSONable

from scs_philips_hue.data.bridge.backup import Backup
from scs_philips_hue.data.bridge.internet_services import InternetServices
from scs_philips_hue.data.bridge.portal_state import PortalState
from scs_philips_hue.data.bridge.sw_update import SWUpdate
from scs_philips_hue.data.bridge.sw_update_2 import SWUpdate2
from scs_philips_hue.data.bridge.whitelist import WhitelistGroup


# --------------------------------------------------------------------------------------------------------------------

class BridgeConfig(JSONable):
    """
    classdocs
    """

    NAME_MIN_LENGTH =        4
    NAME_MAX_LENGTH =       16

    ZIGBEE_CHANNELS =       ("11", "15", "20", "25")


    # ----------------------------------------------------------------------------------------------------------------

    @classmethod
    def construct_from_jdict(cls, jdict):
        if not jdict:
            return None

        name = jdict.get('name')
        zigbee_channel = jdict.get('zigbeechannel')

        dhcp = jdict.get('dhcp')
        ip_address = jdict.get('ipaddress')
        netmask = jdict.get('netmask')
        gateway = jdict.get('gateway')

        proxy_address = jdict.get('proxyaddress')
        proxy_port = jdict.get('proxyport')

        utc = jdict.get('UTC')
        timezone = jdict.get('timezone')

        sw_update = SWUpdate.construct_from_jdict(jdict.get('swupdate'))

        link_button = jdict.get('linkbutton')
        portal_services = jdict.get('portalservices')

        return BridgeConfig(name=name, zigbee_channel=zigbee_channel,
                            dhcp=dhcp, ip_address=ip_address, netmask=netmask, gateway=gateway,
                            proxy_address=proxy_address, proxy_port=proxy_port,
                            utc=utc, timezone=timezone, sw_update=sw_update,
                            link_button=link_button, portal_services=portal_services)


    # ----------------------------------------------------------------------------------------------------------------

    def __init__(self, name=None, zigbee_channel=None,
                 dhcp=None, ip_address=None, netmask=None, gateway=None,
                 proxy_address=None, proxy_port=None,
                 utc=None, timezone=None, sw_update=None,
                 link_button=None, portal_services=None):
        """
        Constructor
        """
        self.__name = name                                                  # string
        self.__zigbee_channel = Datum.int(zigbee_channel)                   # int

        self.__dhcp = Datum.bool(dhcp)                                      # bool
        self.__ip_address = ip_address                                      # string
        self.__netmask = netmask                                            # string
        self.__gateway = gateway                                            # string

        self.__proxy_address = proxy_address                                # string
        self.__proxy_port = Datum.int(proxy_port)                           # int

        self.__utc = utc                                                    # string (date / time)
        self.__timezone = timezone                                          # string (timezone ID)

        self.__sw_update = sw_update                                        # SWUpdate

        self.__link_button = Datum.bool(link_button)                        # bool
        self.__portal_services = Datum.bool(portal_services)                # bool


    # ----------------------------------------------------------------------------------------------------------------

    def as_json(self):
        jdict = OrderedDict()

        if self.name is not None:
            jdict['name'] = self.name

        if self.zigbee_channel is not None:
            jdict['zigbeechannel'] = self.zigbee_channel


        if self.dhcp is not None:
            jdict['dhcp'] = self.dhcp

        if self.ip_address is not None:
            jdict['ipaddress'] = self.ip_address

        if self.netmask is not None:
            jdict['netmask'] = self.netmask

        if self.gateway is not None:
            jdict['gateway'] = self.gateway


        if self.proxy_address is not None:
            jdict['proxyaddress'] = self.proxy_address

        if self.proxy_port is not None:
            jdict['proxyport'] = self.proxy_port


        if self.utc is not None:
            jdict['UTC'] = self.utc

        if self.timezone is not None:
            jdict['timezone'] = self.timezone


        if self.sw_update is not None:
            jdict['swupdate'] = self.sw_update


        if self.link_button is not None:
            jdict['linkbutton'] = self.link_button

        if self.portal_services is not None:
            jdict['portalservices'] = self.portal_services

        return jdict


    # ----------------------------------------------------------------------------------------------------------------

    @property
    def name(self):
        return self.__name


    @property
    def zigbee_channel(self):
        return self.__zigbee_channel


    @property
    def dhcp(self):
        return self.__dhcp


    @property
    def ip_address(self):
        return self.__ip_address


    @property
    def netmask(self):
        return self.__netmask


    @property
    def gateway(self):
        return self.__gateway


    @property
    def proxy_address(self):
        return self.__proxy_address


    @property
    def proxy_port(self):
        return self.__proxy_port


    @property
    def utc(self):
        return self.__utc


    @property
    def timezone(self):
        return self.__timezone


    @property
    def sw_update(self):
        return self.__sw_update


    @property
    def link_button(self):
        return self.__link_button


    @property
    def portal_services(self):
        return self.__portal_services


    # ----------------------------------------------------------------------------------------------------------------

    def __str__(self, *args, **kwargs):
        return "BridgeConfig:{name:%s, zigbee_channel:%s, " \
               "dhcp:%s, ip_address:%s, netmask:%s, " \
               "gateway:%s, proxy_address:%s, proxy_port:%s," \
               "utc:%s, timezone:%s, sw_update:%s," \
               "link_button:%s, portal_services:%s}" %  \
               (self.name, self.zigbee_channel,
                self.dhcp, self.ip_address, self.netmask,
                self.gateway, self.proxy_address, self.proxy_port,
                self.utc, self.timezone, self.sw_update,
                self.link_button, self.portal_services)


# --------------------------------------------------------------------------------------------------------------------

class ReportedBridgeConfig(BridgeConfig):
    """
    classdocs
    """

    # ----------------------------------------------------------------------------------------------------------------

    @classmethod
    def construct_from_jdict(cls, jdict):
        # print(JSONify.dumps(jdict))

        if not jdict:
            return None

        name = jdict.get('name')
        zigbee_channel = jdict.get('zigbeechannel')
        bridge_id = jdict.get('bridgeid')
        mac = jdict.get('mac')

        dhcp = jdict.get('dhcp')
        ip_address = jdict.get('ipaddress')
        netmask = jdict.get('netmask')
        gateway = jdict.get('gateway')

        proxy_address = jdict.get('proxyaddress')
        proxy_port = jdict.get('proxyport')

        utc = jdict.get('UTC')
        localtime = jdict.get('localtime')
        timezone = jdict.get('timezone')

        model_id = jdict.get('modelid')
        datastore_version = jdict.get('datastoreversion')
        sw_version = jdict.get('swversion')
        api_version = jdict.get('apiversion')

        sw_update = SWUpdate.construct_from_jdict(jdict.get('swupdate'))
        sw_update_2 = SWUpdate2.construct_from_jdict(jdict.get('swupdate2'))

        link_button = jdict.get('linkbutton')

        portal_services = jdict.get('portalservices')
        portal_connection = jdict.get('portalconnection')
        portal_state = PortalState.construct_from_jdict(jdict.get('portalstate'))

        internet_services = InternetServices.construct_from_jdict(jdict.get('internetservices'))
        factory_new = jdict.get('factorynew')
        replaces_bridge_id = jdict.get('replacesbridgeid')
        backup = Backup.construct_from_jdict(jdict.get('backup'))
        starterkit_id = jdict.get('starterkitid')

        whitelist = WhitelistGroup.construct_from_jdict(jdict.get('whitelist'))

        return ReportedBridgeConfig(name=name, zigbee_channel=zigbee_channel, bridge_id=bridge_id,
                                    mac=mac, dhcp=dhcp, ip_address=ip_address, netmask=netmask, gateway=gateway,
                                    proxy_address=proxy_address, proxy_port=proxy_port,
                                    utc=utc, localtime=localtime, timezone=timezone,
                                    model_id=model_id, datastore_version=datastore_version, sw_version=sw_version,
                                    api_version=api_version, sw_update=sw_update, sw_update_2=sw_update_2,
                                    link_button=link_button,
                                    portal_services=portal_services, portal_connection=portal_connection,
                                    portal_state=portal_state,
                                    internet_services=internet_services, factory_new=factory_new,
                                    replaces_bridge_id=replaces_bridge_id, backup=backup, starterkit_id=starterkit_id,
                                    whitelist=whitelist)


    # ----------------------------------------------------------------------------------------------------------------

    def __init__(self, name=None, zigbee_channel=None, bridge_id=None,
                 mac=None, dhcp=None, ip_address=None, netmask=None, gateway=None,
                 proxy_address=None, proxy_port=None,
                 utc=None, localtime=None, timezone=None,
                 model_id=None, datastore_version=None, sw_version=None,
                 api_version=None, sw_update=None, sw_update_2=None,
                 link_button=None,
                 portal_services=None, portal_connection=None,
                 portal_state=None,
                 internet_services=None, factory_new=None,
                 replaces_bridge_id=None, backup=None, starterkit_id=None,
                 whitelist=None):
        """
        Constructor
        """
        super().__init__(name=name, zigbee_channel=zigbee_channel,
                         dhcp=dhcp, ip_address=ip_address, netmask=netmask, gateway=gateway,
                         proxy_address=proxy_address, proxy_port=proxy_port,
                         utc=utc, timezone=timezone, sw_update=sw_update,
                         link_button=link_button)

        self.__bridge_id = bridge_id                                        # string
        self.__mac = mac                                                    # string

        self.__localtime = localtime                                        # string (date / time)

        self.__model_id = model_id                                          # string
        self.__datastore_version = datastore_version                        # string

        self.__sw_version = sw_version                                      # string
        self.__api_version = api_version                                    # string

        self.__sw_update_2 = sw_update_2                                    # SWUpdate2

        self.__portal_services = Datum.bool(portal_services)                # bool
        self.__portal_connection = portal_connection                        # string
        self.__portal_state = portal_state                                  # PortalState

        self.__internet_services = internet_services                        # InternetServices
        self.__factory_new = Datum.bool(factory_new)                        # bool
        self.__replaces_bridge_id = replaces_bridge_id                      # string
        self.__backup = backup                                              # Backup
        self.__starterkit_id = starterkit_id                                # string

        self.__whitelist = whitelist                                        # WhitelistGroup


    # ----------------------------------------------------------------------------------------------------------------

    def as_json(self):
        jdict = OrderedDict()

        jdict['name'] = self.name
        jdict['zigbeechannel'] = self.zigbee_channel
        jdict['bridgeid'] = self.bridge_id
        jdict['mac'] = self.mac

        jdict['dhcp'] = self.dhcp
        jdict['ipaddress'] = self.ip_address
        jdict['netmask'] = self.netmask
        jdict['gateway'] = self.gateway

        jdict['proxyaddress'] = self.proxy_address
        jdict['proxyport'] = self.proxy_port

        jdict['UTC'] = self.utc
        jdict['localtime'] = self.localtime
        jdict['timezone'] = self.timezone

        jdict['modelid'] = self.model_id
        jdict['datastoreversion'] = self.datastore_version
        jdict['swversion'] = self.sw_version
        jdict['apiversion'] = self.api_version

        jdict['swupdate'] = self.sw_update
        jdict['swupdate2'] = self.sw_update_2

        jdict['linkbutton'] = self.link_button

        jdict['portalservices'] = self.portal_services
        jdict['portalconnection'] = self.portal_connection
        jdict['portalstate'] = self.portal_state

        jdict['internetservices'] = self.internet_services
        jdict['factorynew'] = self.factory_new
        jdict['replacesbridgeid'] = self.replaces_bridge_id
        jdict['backup'] = self.backup

        jdict['starterkitid'] = self.starterkit_id
        jdict['whitelist'] = self.whitelist

        return jdict


    # ----------------------------------------------------------------------------------------------------------------

    @property
    def bridge_id(self):
        return self.__bridge_id


    @property
    def mac(self):
        return self.__mac


    @property
    def localtime(self):
        return self.__localtime


    @property
    def model_id(self):
        return self.__model_id


    @property
    def datastore_version(self):
        return self.__datastore_version


    @property
    def sw_version(self):
        return self.__sw_version


    @property
    def api_version(self):
        return self.__api_version


    @property
    def sw_update_2(self):
        return self.__sw_update_2


    @property
    def portal_services(self):
        return self.__portal_services


    @property
    def portal_connection(self):
        return self.__portal_connection


    @property
    def portal_state(self):
        return self.__portal_state


    @property
    def internet_services(self):
        return self.__internet_services


    @property
    def factory_new(self):
        return self.__factory_new


    @property
    def replaces_bridge_id(self):
        return self.__replaces_bridge_id


    @property
    def backup(self):
        return self.__backup


    @property
    def starterkit_id(self):
        return self.__starterkit_id


    @property
    def whitelist(self):
        return self.__whitelist


    # ----------------------------------------------------------------------------------------------------------------

    def __str__(self, *args, **kwargs):
        return "ReportedBridgeConfig:{name:%s, zigbee_channel:%s, bridge_id:%s, mac:%s, " \
               "dhcp:%s, ip_address:%s, netmask:%s, " \
               "gateway:%s, proxy_address:%s, proxy_port:%s," \
               "utc:%s, localtime:%s, timezone:%s, " \
               "model_id:%s, datastore_version:%s, sw_version:%s, api_version:%s, " \
               "sw_update:%s, sw_update_2:%s, link_button:%s," \
               "portal_services:%s, portal_connection:%s, portal_state:%s, internet_services:%s, " \
               "factory_new:%s, replaces_bridge_id:%s, backup:%s, starterkit_id:%s, " \
               "whitelist:%s}" %  \
               (self.name, self.zigbee_channel, self.bridge_id, self.mac, self.dhcp, self.ip_address, self.netmask,
                self.gateway, self.proxy_address, self.proxy_port,
                self.utc, self.localtime, self.timezone,
                self.model_id, self.datastore_version, self.sw_version, self.api_version,
                self.sw_update, self.sw_update_2, self.link_button,
                self.portal_services, self.portal_connection, self.portal_state, self.internet_services,
                self.factory_new, self.replaces_bridge_id, self.backup, self.starterkit_id,
                self.whitelist)



#!/usr/bin/env python3

"""
Created on 11 Apr 2017

@author: Bruno Beloff (bruno.beloff@southcoastscience.com)

source repo: scs_philips_hue

DESCRIPTION
The uds_receiver utility is used to accept data via a Unix domain socket, with data sourced from the same host, or
another host on the same local area network.

SYNOPSIS
uds_receiver.py [-v] UDS_SUB

EXAMPLES
./uds_receiver.py scs-particulates.uds

SEE ALSO
scs_analysis/socket_receiver
"""

import sys

from scs_core.comms.uds_reader import UDSReader

from scs_core.sys.logging import Logging
from scs_core.sys.signalled_exit import SignalledExit

from scs_host.comms.domain_socket import DomainSocket

from scs_philips_hue.cmd.cmd_uds import CmdUDS


# --------------------------------------------------------------------------------------------------------------------

if __name__ == '__main__':

    # ----------------------------------------------------------------------------------------------------------------
    # cmd...

    cmd = CmdUDS()

    if not cmd.is_valid():
        cmd.print_help(sys.stderr)
        exit(2)

    Logging.config('uds_receiver', verbose=cmd.verbose)
    logger = Logging.getLogger()

    logger.info(cmd)


    # ----------------------------------------------------------------------------------------------------------------
    # resources...

    uds = UDSReader(DomainSocket, cmd.path)
    logger.info(uds)

    try:
        # ------------------------------------------------------------------------------------------------------------
        # run...

        # signal handler...
        SignalledExit.construct("uds_receiver", cmd.verbose)

        uds.connect()

        for message in uds.messages():
            print(message)
            sys.stdout.flush()


    # ----------------------------------------------------------------------------------------------------------------
    # end...

    except (KeyboardInterrupt, SystemExit):
        pass

    finally:
        logger.info("finishing")
        uds.close()



"""
Created on 16 Mar 2018

@author: Bruno Beloff (bruno.beloff@southcoastscience.com)

document example:
{"lamp-channels": ["scs-hcl-001", "scs-hcl-002"]}
"""

import optparse

from scs_philips_hue import version


# --------------------------------------------------------------------------------------------------------------------

class CmdDeskConf(object):
    """unix command line handler"""

    def __init__(self):
        """
        Constructor
        """
        self.__parser = optparse.OptionParser(usage="%prog [-c CHANNEL { -a LAMP_NAME | -r LAMP_NAME | -d }] "
                                                    "[-i INDENT] [-v]", version=version())

        # configuration...
        self.__parser.add_option("--channel", "-c", type="string", action="store", dest="channel",
                                 help="the name of the information channel")

        # operations...
        self.__parser.add_option("--add", "-a", type="string", action="store", dest="add_lamp",
                                 help="add the given lamp")

        self.__parser.add_option("--remove", "-r", type="string", action="store", dest="remove_lamp",
                                 help="remove the given lamp")

        self.__parser.add_option("--delete", "-d", action="store_true", dest="delete",
                                 help="delete the desk configuration")

        # output...
        self.__parser.add_option("--indent", "-i", type="int", action="store", dest="indent",
                                 help="pretty-print the output with INDENT")

        self.__parser.add_option("--verbose", "-v", action="store_true", dest="verbose", default=False,
                                 help="report narrative to stderr")

        self.__opts, self.__args = self.__parser.parse_args()


    # ----------------------------------------------------------------------------------------------------------------

    def is_valid(self):
        if self.add_lamp is not None and self.remove_lamp is not None:
            return False

        if self.set() and self.channel is None:
            return False

        if not self.set() and self.channel is not None:
            return False

        return True


    def set(self):
        return self.add_lamp is not None or self.remove_lamp is not None


    # ----------------------------------------------------------------------------------------------------------------

    @property
    def channel(self):
        return self.__opts.channel


    @property
    def add_lamp(self):
        return self.__opts.add_lamp


    @property
    def remove_lamp(self):
        return self.__opts.remove_lamp


    @property
    def delete(self):
        return self.__opts.delete


    @property
    def indent(self):
        return self.__opts.indent


    @property
    def verbose(self):
        return self.__opts.verbose


    # ----------------------------------------------------------------------------------------------------------------

    def print_help(self, file):
        self.__parser.print_help(file)


    def __str__(self, *args, **kwargs):
        return "CmdDeskConf:{channel:%s, add_lamp:%s, remove_lamp:%s, delete:%s, indent:%s, verbose:%s}" % \
                    (self.channel, self.add_lamp, self.remove_lamp, self.delete, self.indent, self.verbose)



#!/usr/bin/env python3

"""
Created on 4 Nov 2017

@author: Bruno Beloff (bruno.beloff@southcoastscience.com)

source repo: scs_philips_hue

DESCRIPTION
The user utility is used to manage user accounts or "whitelist entries" on a Philips Hue Bridge device.

Note that the user JSON document presented by the user utility has a description field of the form
app#user-name. When a delete command is performed, only the user-name component should be used to identify
the user.

SYNOPSIS
user.py { -l | -r USER } [-i INDENT] [-v] BRIDGE_NAME

EXAMPLES
./user.py -vi4 -l hue-br1-001

FILES
~/SCS/hue/bridge_address_set.json
~/SCS/hue/bridge_credentials_set.json

DOCUMENT EXAMPLE - OUTPUT
[{"username": "JuA71dSBcObSJi255VatGFEQxOFDu6lOzDqCtjtI", "last use date": "2023-04-21T10:21:34",
"create date": "2023-04-21T08:10:33", "description": "scs-hue-connector#bruno16"}]


SEE ALSO
scs_philips_hue/join
scs_philips_hue/bridge
scs_philips_hue/desk
"""

import sys

from scs_core.client.http_exception import HTTPException
from scs_core.client.resource_unavailable_exception import ResourceUnavailableException

from scs_core.data.json import JSONify

from scs_core.sys.logging import Logging

from scs_host.sys.host import Host

from scs_philips_hue.cmd.cmd_user import CmdUser

from scs_philips_hue.config.bridge_credentials import BridgeCredentialsSet

from scs_philips_hue.manager.bridge_builder import BridgeBuilder
from scs_philips_hue.manager.user_manager import UserManager


# --------------------------------------------------------------------------------------------------------------------

if __name__ == '__main__':

    application_key = ''

    bridge = None

    # ----------------------------------------------------------------------------------------------------------------
    # cmd...

    cmd = CmdUser()

    if not cmd.is_valid():
        cmd.print_help(sys.stderr)
        exit(2)

    Logging.config('user', verbose=cmd.verbose)
    logger = Logging.getLogger()

    logger.info(cmd)

    try:

        # -------------------------------------------------------------------------------------------------------------
        # resource...


        # credentials...
        credentials_set = BridgeCredentialsSet.load(Host, skeleton=True)

        try:
            credentials = credentials_set.credentials(cmd.bridge_name)
            logger.info(credentials)

        except KeyError:
            logger.error("no stored credentials for bridge '%s'." % cmd.bridge_name)
            exit(1)

        # manager...
        bridge_manager = BridgeBuilder(Host).construct_manager_for_credentials(credentials)
        bridge = bridge_manager.find()

        # manager...
        manager = UserManager(bridge.ip_address, credentials.username)


        # ------------------------------------------------------------------------------------------------------------
        # run...

        users = manager.find_all()

        if cmd.remove:
            for user in users:
                if user.description.user == cmd.remove:
                    response = manager.delete(user.username)
                    logger.info(response)

        else:
            print(JSONify.dumps(users, indent=cmd.indent))


    # ----------------------------------------------------------------------------------------------------------------
    # end...

    except KeyboardInterrupt:
        print(file=sys.stderr)

    except (ConnectionError, HTTPException, ResourceUnavailableException) as ex:
        logger.error(repr(ex))
        exit(1)



#!/usr/bin/env python3

"""
Created on 16 Mar 2018

@author: Bruno Beloff (bruno.beloff@southcoastscience.com)

source repo: scs_philips_hue

DESCRIPTION
The domain_conf utility is used to specify an environmental data messaging topic path, plus a leaf node within
documents delivered on that messaging topic. Together, these two parameters yield a scalar value that can be
accepted by a mapping utility such as chroma.

The domain_conf.json document managed by the domain_conf utility is used by the aws_mqtt_subscriber and node utilities.

SYNOPSIS
domain_conf.py [-c CHANNEL { -a TOPIC_PATH DOMAIN_NODE | -r }] [-i INDENT] [-v]

EXAMPLES
./domain_conf.py -vi4 -c assembly-co2 -a south-coast-science-production/freshfield-environment/loc/1/gases val.CO2.cnc

FILES
~/SCS/hue/domain_conf_set.json

DOCUMENT EXAMPLE
{"NO2": {"topic-path": "south-coast-science-demo/brighton/loc/1/gases", "document-node": "exg.val.NO2.cnc"},
"PM10": {"topic-path": "south-coast-science-demo/brighton/loc/1/particulates", "document-node": "exg.val.pm10"}}

SEE ALSO
scs_philips_hue/aws_mqtt_subscriber
scs_philips_hue/node
"""

import sys

from scs_core.data.json import JSONify
from scs_core.sys.logging import Logging

from scs_host.sys.host import Host

from scs_philips_hue.cmd.cmd_domain_conf import CmdDomainConf
from scs_philips_hue.config.domain_conf import DomainConfSet


# --------------------------------------------------------------------------------------------------------------------

if __name__ == '__main__':

    # ----------------------------------------------------------------------------------------------------------------
    # cmd...

    cmd = CmdDomainConf()

    if not cmd.is_valid():
        cmd.print_help(sys.stderr)
        exit(2)

    Logging.config('domain_conf', verbose=cmd.verbose)
    logger = Logging.getLogger()

    logger.info(cmd)


    # ----------------------------------------------------------------------------------------------------------------
    # resources...

    # ChromaConf...
    domains = DomainConfSet.load(Host, skeleton=True)


    # ----------------------------------------------------------------------------------------------------------------
    # run...

    if cmd.add:
        domains.add(cmd.channel, cmd.add[0], cmd.add[1])
        domains.save(Host)

    if cmd.remove:
        domains.remove(cmd.channel)
        domains.save(Host)

    if domains:
        print(JSONify.dumps(domains, indent=cmd.indent))



"""
Created on 28 Oct 2017

@author: Bruno Beloff (bruno.beloff@southcoastscience.com)

example:
{"signedon": true, "incoming": false, "outgoing": true, "communication": "disconnected"}
"""

from collections import OrderedDict

from scs_core.data.json import JSONable


# --------------------------------------------------------------------------------------------------------------------

class PortalState(JSONable):
    """
    classdocs
    """

    # ----------------------------------------------------------------------------------------------------------------

    @classmethod
    def construct_from_jdict(cls, jdict):
        if not jdict:
            return None

        signed_on = jdict.get('signedon')
        incoming = jdict.get('incoming')
        outgoing = jdict.get('outgoing')

        communication = jdict.get('communication')

        return PortalState(signed_on, incoming, outgoing, communication)


    # ----------------------------------------------------------------------------------------------------------------

    def __init__(self, signed_on, incoming, outgoing, communication):
        """
        Constructor
        """
        self.__signed_on = bool(signed_on)                  # bool
        self.__incoming = bool(incoming)                    # bool
        self.__outgoing = bool(outgoing)                    # bool

        self.__communication = communication                # string


    # ----------------------------------------------------------------------------------------------------------------

    def as_json(self):
        jdict = OrderedDict()

        jdict['signedon'] = self.signed_on
        jdict['incoming'] = self.incoming
        jdict['outgoing'] = self.outgoing

        jdict['communication'] = self.communication

        return jdict


    # ----------------------------------------------------------------------------------------------------------------

    @property
    def signed_on(self):
        return self.__signed_on


    @property
    def incoming(self):
        return self.__incoming


    @property
    def outgoing(self):
        return self.__outgoing


    @property
    def communication(self):
        return self.__communication


    # ----------------------------------------------------------------------------------------------------------------

    def __str__(self, *args, **kwargs):
        return "PortalState:{signed_on:%s, incoming:%s, outgoing:%s, communication:%s}" %  \
               (self.signed_on, self.incoming, self.outgoing, self.communication)
